@@ -1,8 +1,8 @@
 /*
  *  Hamlib Alinco backend - main file
- *  Copyright (c) 2001 by Stephane Fillod
+ *  Copyright (c) 2001-2003 by Stephane Fillod
  *
- *		$Id: alinco.c,v 1.7 2001/07/14 16:48:03 f4cfe Exp $
+ *	$Id: alinco.c,v 1.16 2003/04/16 22:30:38 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -25,34 +25,23 @@
 #endif
 
 #include <stdlib.h>
-#include <stdio.h>   /* Standard input/output definitions */
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
-#include <fcntl.h>   /* File control definitions */
-#include <errno.h>   /* Error number definitions */
-#include <termios.h> /* POSIX terminal control definitions */
-#include <sys/ioctl.h>
 #include <math.h>
 
-#if defined(__CYGWIN__)
-#  undef HAMLIB_DLL
-#  include <hamlib/rig.h>
-#  include <serial.h>
-#  include <misc.h>
-#  include <cal.h>
-#  define HAMLIB_DLL
-#  include <hamlib/rig_dll.h>
-#else
-#  include <hamlib/rig.h>
-#  include <serial.h>
-#  include <misc.h>
-#  include <cal.h>
-#endif
+#include <hamlib/rig.h>
+#include <serial.h>
+#include <misc.h>
+#include <cal.h>
+#include <register.h>
 
 #include "alinco.h"
 
 
-#define EOM "\r"
+/* Line Feed */
+#define EOM "\x0a"
+
+#define BUFSZ 32
 
 /*
  * modes in use by the "2G" command
@@ -108,11 +97,13 @@
  */
 int alinco_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *data_len)
 {
-	int i, count, retval;
+	int retval;
 	struct rig_state *rs;
 
 	rs = &rig->state;
 
+	serial_flush(&rs->rigport);
+
 	retval = write_block(&rs->rigport, cmd, cmd_len);
 	if (retval != RIG_OK)
 			return retval;
@@ -120,20 +111,8 @@ int alinco_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *
 	/* no data expected, TODO: flush input? */
 	if (!data || !data_len)
 			return 0;
-	/*
-	 * buffered read are quite helpful here!
-	 * However, an automate with a state model would be more efficient..
-	 */
-	i = 0;
-	do {
-		count = fread_block(&rs->rigport, data+i, 1);
-		if (count > 0)
-				i += count;
-		else if (count < 0)
-				return count;
-	} while (count > 0);
 
-	*data_len = i;
+	*data_len = read_string(&rs->rigport, data, BUFSZ, EOM, strlen(EOM));
 
 	return RIG_OK;
 }
@@ -144,7 +123,7 @@ int alinco_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *
  */
 int alinco_set_vfo(RIG *rig, vfo_t vfo)
 {
-		char cmdbuf[16];
+		char cmdbuf[BUFSZ];
 		int cmd_len;
 		char vfo_num;
 
@@ -170,7 +149,7 @@ int alinco_set_vfo(RIG *rig, vfo_t vfo)
  */
 int alinco_get_vfo(RIG *rig, vfo_t *vfo)
 {
-		char vfobuf[16];
+		char vfobuf[BUFSZ];
 		int vfo_len, retval;
 
 		retval = alinco_transaction(rig, AL CMD_RMV EOM, 
@@ -205,7 +184,7 @@ int alinco_get_vfo(RIG *rig, vfo_t *vfo)
  */
 int alinco_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 {
-		char freqbuf[16];
+		char freqbuf[BUFSZ];
 		int freq_len;
 
 		/* max 10 digits */
@@ -245,7 +224,7 @@ static int current_data_read(RIG *rig, char *databuf)
 int alinco_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 {
 		int retval;
-		char freqbuf[32];
+		char freqbuf[BUFSZ];
 
 		retval = current_data_read(rig, freqbuf);
 		if (retval != RIG_OK)
@@ -253,7 +232,7 @@ int alinco_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 
 		/* extract RX freq */
 		freqbuf[16] = '\0';
-		sscanf(freqbuf+6, "%Ld", freq);
+		sscanf(freqbuf+6, "%lld", freq);
 
 		return RIG_OK;
 }
@@ -264,7 +243,7 @@ int alinco_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
  */
 int alinco_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 {
-		char mdbuf[16];
+		char mdbuf[BUFSZ];
 		int mdbuf_len, narrow_filter, retval;
 		char amode;
 
@@ -311,7 +290,7 @@ int alinco_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 {
 		int retval;
 		int settings;
-		char modebuf[32];
+		char modebuf[BUFSZ];
 
 		retval = current_data_read(rig, modebuf);
 		if (retval != RIG_OK)
@@ -349,10 +328,10 @@ int alinco_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
  * alinco_set_split
  * Assumes rig!=NULL
  */
-int alinco_set_split(RIG *rig, vfo_t vfo, split_t split)
+int alinco_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
 {
 		int cmd_len;
-		char cmdbuf[32];
+		char cmdbuf[BUFSZ];
 
 		cmd_len = sprintf(cmdbuf, AL CMD_SPLT "%d" EOM, 
 						split==RIG_SPLIT_ON ? 1 : 0);
@@ -364,10 +343,10 @@ int alinco_set_split(RIG *rig, vfo_t vfo, split_t split)
  * alinco_get_split
  * Assumes rig!=NULL, split!=NULL
  */
-int alinco_get_split(RIG *rig, vfo_t vfo, split_t *split)
+int alinco_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
 {
 		int splt_len, retval;
-		char spltbuf[32];
+		char spltbuf[BUFSZ];
 
 		retval = alinco_transaction (rig, AL CMD_RSPLT EOM, 
 						strlen(AL CMD_RSPLT EOM), spltbuf, &splt_len);
@@ -399,7 +378,7 @@ int alinco_get_split(RIG *rig, vfo_t vfo, split_t *split)
  */
 int alinco_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 {
-		char freqbuf[16];
+		char freqbuf[BUFSZ];
 		int freq_len;
 		int retval;
 
@@ -422,7 +401,7 @@ int alinco_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 int alinco_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 {
 		int retval;
-		char freqbuf[32];
+		char freqbuf[BUFSZ];
 
 		retval = current_data_read(rig, freqbuf);
 		if (retval != RIG_OK)
@@ -430,7 +409,7 @@ int alinco_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 
 		/* extract TX freq first, as RX kills freqbuf[16] */
 		freqbuf[26] = '\0';
-		sscanf(freqbuf+16, "%Ld", tx_freq);
+		sscanf(freqbuf+16, "%lld", tx_freq);
 
 		return RIG_OK;
 }
@@ -443,7 +422,7 @@ int alinco_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 int alinco_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
 {
 		int rit_len, retval;
-		char ritbuf[32];
+		char ritbuf[BUFSZ];
 
 		/* read in Hertz unit */
 		retval = alinco_transaction (rig, AL CMD_RIT "0" EOM,
@@ -471,7 +450,7 @@ int alinco_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
 int alinco_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
 {
 		int cmd_len;
-		char cmdbuf[32];
+		char cmdbuf[BUFSZ];
 
 		/* Optimize:
 		 *   sort the switch cases with the most frequent first
@@ -514,7 +493,7 @@ int alinco_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
 {
 		int retval;
 		int settings;
-		char funcbuf[32];
+		char funcbuf[BUFSZ];
 
 		/* Optimize:
 		 *   sort the switch cases with the most frequent first
@@ -558,12 +537,12 @@ int alinco_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
 /*
  * alinco_set_level
  * Assumes rig!=NULL
- * FIXME: cannot support PREAMP and ATT both at same time (make sens though)
+ * FIXME: cannot support PREAMP and ATT both at same time (make sense though)
  */
 int alinco_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 {
 		int cmd_len, lvl;
-		char cmdbuf[32];
+		char cmdbuf[BUFSZ];
 
 		/* Optimize:
 		 *   sort the switch cases with the most frequent first
@@ -616,7 +595,7 @@ int alinco_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 {
  		struct alinco_priv_caps *priv;
 		int retval, lvl_len;
-		char lvlbuf[32];
+		char lvlbuf[BUFSZ];
 
 		priv = (struct alinco_priv_caps*)rig->caps->priv;
 
@@ -703,7 +682,7 @@ int alinco_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 int alinco_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
 {
 	const struct rig_caps *caps;
-	unsigned char tonebuf[16];
+	unsigned char tonebuf[BUFSZ];
 	int tone_len;
 	int i;
 								 
@@ -728,7 +707,7 @@ int alinco_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
  */
 int alinco_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 {
-		char pttbuf[16];
+		char pttbuf[BUFSZ];
 		int ptt_len, retval;
 
 		retval = alinco_transaction (rig, AL CMD_PTT EOM, 
@@ -762,7 +741,7 @@ int alinco_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
  */
 int alinco_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
 {
-		char dcdbuf[16];
+		char dcdbuf[BUFSZ];
 		int dcd_len, retval;
 
 		retval = alinco_transaction (rig, AL CMD_SQL EOM,
@@ -796,7 +775,7 @@ int alinco_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
  */
 int alinco_set_mem(RIG *rig, vfo_t vfo, int ch)
 {
-		char cmdbuf[16];
+		char cmdbuf[BUFSZ];
 		int cmd_len;
 
 		if (ch < 0 || ch > 99)
@@ -813,7 +792,7 @@ int alinco_set_mem(RIG *rig, vfo_t vfo, int ch)
  */
 int alinco_get_mem(RIG *rig, vfo_t vfo, int *ch)
 {
-		char membuf[16];
+		char membuf[BUFSZ];
 		int mem_len, retval;
 
 		retval = alinco_transaction (rig, AL CMD_RMEM EOM, 
@@ -840,9 +819,9 @@ int alinco_get_mem(RIG *rig, vfo_t vfo, int *ch)
 
 
 /*
- * init_alinco is called by rig_backend_load
+ * initrigs_alinco is called by rig_backend_load
  */
-int init_alinco(void *be_handle)
+DECLARE_INITRIG_BACKEND(alinco)
 {
 		rig_debug(RIG_DEBUG_VERBOSE, "alinco: _init called\n");
 
