@@ -1,8 +1,8 @@
 /*
  *  Hamlib AOR backend - main header
- *  Copyright (c) 2000,2001 by Stephane Fillod
+ *  Copyright (c) 2000-2003 by Stephane Fillod
  *
- *		$Id: aor.h,v 1.6 2001/07/13 19:08:15 f4cfe Exp $
+ *	$Id: aor.h,v 1.14 2003/04/16 22:30:39 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -23,6 +23,7 @@
 #ifndef _AOR_H
 #define _AOR_H 1
 
+#include <hamlib/rig.h>
 
 int aor_close(RIG *rig);
 
@@ -30,14 +31,21 @@ int aor_set_freq(RIG *rig, vfo_t vfo, freq_t freq);
 int aor_get_freq(RIG *rig, vfo_t vfo, freq_t *freq);
 int aor_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width);
 int aor_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width);
+int aor_set_vfo(RIG *rig, vfo_t vfo);
+int aor_get_vfo(RIG *rig, vfo_t *vfo);
+
+int aor_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val);
+int aor_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val);
 
 int aor_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts);
 int aor_set_powerstat(RIG *rig, powerstat_t status);
+int aor_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op);
+const char *aor_get_info(RIG *rig);
 
 extern const struct rig_caps ar8200_caps;
-
-extern HAMLIB_EXPORT(int) init_aor(void *be_handle);
-
+extern const struct rig_caps ar8000_caps;
+extern const struct rig_caps ar5000_caps;
+extern const struct rig_caps ar3000a_caps;
 
 #endif /* _AOR_H */
 
