@@ -1,8 +1,8 @@
 /*
  *  Hamlib CI-V backend - low level communication routines
- *  Copyright (c) 2000,2001 by Stephane Fillod
+ *  Copyright (c) 2000-2003 by Stephane Fillod
  *
- *		$Id: frame.c,v 1.14 2001/07/13 19:08:15 f4cfe Exp $
+ *	$Id: frame.c,v 1.21 2003/04/26 09:54:49 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -25,25 +25,16 @@
 #endif
 
 #include <stdlib.h>
-#include <stdio.h>   /* Standard input/output definitions */
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
-#include <fcntl.h>   /* File control definitions */
-#include <errno.h>   /* Error number definitions */
-#include <termios.h> /* POSIX terminal control definitions */
-#include <sys/ioctl.h>
 
-#include <hamlib/rig.h>
-#include <hamlib/riglist.h>
-#include <serial.h>
-#include <misc.h>
+#include "hamlib/rig.h"
+#include "serial.h"
+#include "misc.h"
 #include "icom.h"
 #include "icom_defs.h"
 #include "frame.h"
 
-/* Prototypes */
-int read_icom_block(port_t *p, unsigned char *rxbuffer, size_t count);
-
 /*
  * Build a CI-V frame.
  * The whole frame is placed in frame[],
@@ -64,7 +55,9 @@ int make_cmd_frame(char frame[], char re_id, char cmd, int subcmd, const char *d
 {	
 	int i = 0;
 
+#if 0
 	frame[i++] = PAD;	/* give old rigs a chance to flush their rx buffers */
+#endif
 	frame[i++] = PR;	/* Preamble code */
 	frame[i++] = PR;
 	frame[i++] = re_id;
@@ -98,20 +91,24 @@ int icom_transaction (RIG *rig, int cmd, int subcmd, const char *payload, int pa
 {
 		struct icom_priv_data *priv;
 		struct rig_state *rs;
-		unsigned char buf[16];
+		unsigned char buf[MAXFRAMELEN];
+		unsigned char sendbuf[MAXFRAMELEN];
 		int frm_len, retval;
 
 		rs = &rig->state;
 		priv = (struct icom_priv_data*)rs->priv;
 
-		frm_len = make_cmd_frame(buf, priv->re_civ_addr, cmd, subcmd, 
+		frm_len = make_cmd_frame(sendbuf, priv->re_civ_addr, cmd, subcmd, 
 						payload, payload_len);
 
 		/* 
 		 * should check return code and that write wrote cmd_len chars! 
 		 */
 		Hold_Decode(rig);
-		retval = write_block(&rs->rigport, buf, frm_len);
+
+		serial_flush(&rs->rigport);
+
+		retval = write_block(&rs->rigport, sendbuf, frm_len);
 		if (retval != RIG_OK) {
 				Unhold_Decode(rig);
 				return retval;
@@ -126,11 +123,52 @@ int icom_transaction (RIG *rig, int cmd, int subcmd, const char *payload, int pa
 		 * 			up to rs->retry times.
 		 */
 
-		retval = read_icom_block(&rs->rigport, buf, frm_len);
-		if (retval != frm_len) {
-				Unhold_Decode(rig);
-				return retval < 0 ? retval : -RIG_EPROTO;
-		}
+		retval = read_icom_frame(&rs->rigport, buf);
+		if (retval == -RIG_ETIMEOUT || retval == 0)
+		  {
+		    /* Nothing recieved, CI-V interface is not echoing */
+		    Unhold_Decode(rig);
+		    return -RIG_BUSERROR;
+		  }
+		if (retval < 0)
+		  {
+		    /* Other error, return it */
+		    Unhold_Decode(rig);
+		    return retval;
+		  }
+
+		switch (buf[retval-1])
+		  {
+		  case COL:
+		    /* Collision */
+		    Unhold_Decode(rig);
+		    return -RIG_BUSBUSY;
+		  case FI:
+		    /* Ok, normal frame */
+		    break;
+		  default:
+		    /* Timeout after reading at least one character */
+		    /* Problem on ci-v bus? */
+		    Unhold_Decode(rig);
+		    return -RIG_BUSERROR;
+		  }
+
+		if (retval != frm_len) 
+		  {
+		    /* Not the same length??? */
+		    /* Problem on ci-v bus? */
+		    /* Someone else got a packet in? */
+		    Unhold_Decode(rig);
+		    return -RIG_EPROTO;
+		  }
+		if (memcmp(buf,sendbuf,frm_len))
+		  {
+		    /* Frames are different? */
+		    /* Problem on ci-v bus? */
+		    /* Someone else got a packet in? */
+		    Unhold_Decode(rig);
+		    return -RIG_EPROTO;
+		  }
 
 		/*
 		 * wait for ACK ... 
@@ -139,10 +177,30 @@ int icom_transaction (RIG *rig, int cmd, int subcmd, const char *payload, int pa
 		 */
 		frm_len = read_icom_frame(&rs->rigport, buf);
 		Unhold_Decode(rig);
-		if (frm_len < 0) {
-				return frm_len;
-		} else if (frm_len < ACKFRMLEN) {
-				return  -RIG_EPROTO;
+		
+		if (frm_len < 0) 
+		  {
+		    /* RIG_TIMEOUT: timeout getting response, return timeout */
+		    /* other error: return it */
+		    return frm_len;
+		  }
+		
+		switch (buf[frm_len-1])
+		  {
+		  case COL:
+		    /* Collision */
+		    return -RIG_BUSBUSY;
+		  case FI:
+		    /* Ok, normal frame */
+		    break;
+		  default:
+		    /* Timeout after reading at least one character */
+		    /* Problem on ci-v bus? */
+		    return  -RIG_EPROTO;
+		  }
+
+		if (frm_len < ACKFRMLEN) {
+		  return  -RIG_EPROTO;
 		}
 
 		*data_len = frm_len-(ACKFRMLEN-1);
@@ -151,33 +209,13 @@ int icom_transaction (RIG *rig, int cmd, int subcmd, const char *payload, int pa
 		/*
 		 * TODO: check addresses in reply frame
 		 */
-
+		
 		return RIG_OK;
 }
 
-/*
- * read count useful bytes, discarding collisions
- * FIXME: check return codes/bytes read
- * this function will be deprecated soon!
- */
-int read_icom_block(port_t *p, unsigned char *rxbuffer, size_t count)
-{
-		int i;
-
-		i = fread_block(p, rxbuffer, count);
-
-#if 0
-		for (i=0; i<count; i++) {
-			if (rxbuffer[i] != COL)
-				break;
-		}
-		if (i > 0) {
-			memmove(rxbuffer, rxbuffer+i, count-i);
-			fread_block(stream, rxbuffer+i, count-i, timeout);
-		}
-#endif
-		return i;	/* duh! */
-}
+/* used in read_icom_frame as end of block */
+static const char icom_block_end[2] = {FI, COL};
+#define icom_block_end_length 2
 
 /*
  * read_icom_frame
@@ -187,41 +225,34 @@ int read_icom_block(port_t *p, unsigned char *rxbuffer, size_t count)
  */
 int read_icom_frame(port_t *p, unsigned char rxbuffer[])
 {
-		int i, count;
+		int i;
 
-		/*
-		 * ACKFRMLEN is supposed to be the smallest frame
-		 * we can expected on the CI-V bus
-		 * FIXME: a COLlision is smaller!!
-		 */
-		count = fread_block(p, rxbuffer, ACKFRMLEN);
-		if (count != ACKFRMLEN)
-				return count;
-
-		/*
-		 * buffered read are quite helpful here!
-		 * However, an automate with a state model would be more efficient..
-		 */
-		for (i=ACKFRMLEN; rxbuffer[i-1]!=FI; i++) {
-			count = fread_block(p, rxbuffer+i, 1);
-			if (count != 1)
-					return i;
-		}
+		i = read_string(p, rxbuffer, MAXFRAMELEN, 
+						icom_block_end, icom_block_end_length);
 
 		return i;
 }
 
 
 /*
+ * convert mode and width as expressed by Hamlib frontend
+ * to mode and passband data understandable by a CI-V rig
+ *
+ * if pd == -1, no passband data is to be sent
+ *
+ * return RIG_OK if everything's fine, negative value otherwise
+ *
  * TODO: be more exhaustive
  * assumes rig!=NULL
  */
-unsigned short rig2icom_mode(RIG *rig, rmode_t mode, pbwidth_t width)
+int rig2icom_mode(RIG *rig, rmode_t mode, pbwidth_t width, 
+				unsigned char *md, char *pd)
 {
-		int icmode, icmode_ext;
-		pbwidth_t norm_width;
+		unsigned char icmode;
+		signed char icmode_ext;
+		pbwidth_t medium_width;
 
-		icmode_ext = 0;
+		icmode_ext = -1;
 
 		switch (mode) {
 		case RIG_MODE_AM:	icmode = S_AM; break;
@@ -233,31 +264,49 @@ unsigned short rig2icom_mode(RIG *rig, rmode_t mode, pbwidth_t width)
 		case RIG_MODE_WFM:	icmode = S_WFM; break;
 		default:
 			rig_debug(RIG_DEBUG_ERR,"icom: Unsupported Hamlib mode %d\n",mode);
-			icmode = 0xff;
+			return -RIG_EINVAL;
 		}
 
-		norm_width = rig_passband_normal(rig, mode);
-		if (width == norm_width || width == RIG_PASSBAND_NORMAL)
-				icmode_ext = 0x00;
-		else if (width < norm_width)
-				icmode_ext = 0x02;
+		medium_width = rig_passband_normal(rig, mode);
+		if (width == medium_width || width == RIG_PASSBAND_NORMAL)
+				icmode_ext = -1;	/* medium, no passband data */
+		else if (width < medium_width)
+				icmode_ext = PD_NARROW;
 		else
-				icmode_ext = 0x01;
+				icmode_ext = PD_WIDE;
 
-		return (icmode_ext<<8 | icmode);
+		if (rig->caps->rig_model == RIG_MODEL_ICR7000) {
+				if (mode == RIG_MODE_USB || mode == RIG_MODE_LSB) {
+						icmode = S_R7000_SSB;
+						icmode_ext = 0x00;
+				} else if (mode == RIG_MODE_AM && icmode_ext == -1) {
+						icmode_ext = PD_WIDE;	/* default to Wide */
+				}
+		}
+		
+		*md = icmode;
+		*pd = icmode_ext;
+		return RIG_OK;
 }
 
 /*
  * assumes rig!=NULL, mode!=NULL, width!=NULL
  */
-void icom2rig_mode(RIG *rig, unsigned short icmode, rmode_t *mode, pbwidth_t *width)
+void icom2rig_mode(RIG *rig, unsigned char md, int pd, rmode_t *mode, pbwidth_t *width)
 {
 		*width = RIG_PASSBAND_NORMAL;
 
-		switch (icmode & 0xff) {
+		switch (md) {
 		case S_AM:	*mode = RIG_MODE_AM; break;
 		case S_CW:	*mode = RIG_MODE_CW; break;
-		case S_FM:	*mode = RIG_MODE_FM; break;
+		case S_FM:	if (rig->caps->rig_model == RIG_MODEL_ICR7000
+									&& pd == 0x00) {
+							*mode = RIG_MODE_USB;
+							*width = rig_passband_normal(rig, RIG_MODE_USB);
+							return;
+					} else
+							*mode = RIG_MODE_FM;
+					break;
 		case S_WFM:	*mode = RIG_MODE_WFM; break;
 		case S_USB:	*mode = RIG_MODE_USB; break;
 		case S_LSB:	*mode = RIG_MODE_LSB; break;
@@ -266,16 +315,17 @@ void icom2rig_mode(RIG *rig, unsigned short icmode, rmode_t *mode, pbwidth_t *wi
 
 		default:
 			rig_debug(RIG_DEBUG_ERR,"icom: Unsupported Icom mode %#.2x\n",
-							icmode);
+							md);
 			*mode = RIG_MODE_NONE;
 		}
-		switch ((icmode>>8) & 0xff) {
+		switch (pd) {
 			case 0x00: *width = rig_passband_narrow(rig, *mode); break;
 			case 0x01: *width = rig_passband_normal(rig, *mode); break;
 			case 0x02: *width = rig_passband_wide(rig, *mode); break;
+			case -1: break;		/* no passband data */
 		default:
 			rig_debug(RIG_DEBUG_ERR,"icom: Unsupported Icom mode width %#.2x\n",
-							icmode);
+							pd);
 			*width = RIG_PASSBAND_NORMAL;
 		}
 }
