@@ -1,8 +1,8 @@
 /*
  *  Hamlib CI-V backend - low level communication header
- *  Copyright (c) 2000,2001 by Stephane Fillod
+ *  Copyright (c) 2000-2003 by Stephane Fillod
  *
- *		$Id: frame.h,v 1.9 2001/09/19 21:17:35 f4cfe Exp $
+ *	$Id: frame.h,v 1.13 2003/04/26 09:54:49 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -23,18 +23,18 @@
 #ifndef _FRAME_H
 #define _FRAME_H 1
 
+#define MAXFRAMELEN 32
 
 /*
  * helper functions 
  */
 int make_cmd_frame(char frame[], char re_id, char cmd, int subcmd, const char *data, int data_len);
 
-int read_icom_block(port_t *p, unsigned char *rxbuffer, size_t count);
 int icom_transaction (RIG *rig, int cmd, int subcmd, const char *payload, int payload_len, char *data, int *data_len);
 int read_icom_frame(port_t *p, unsigned char rxbuffer[]);
 
-unsigned short rig2icom_mode(RIG *rig, rmode_t mode, pbwidth_t width);
-void icom2rig_mode(RIG *rig, unsigned short icmode, rmode_t *mode, pbwidth_t *width);
+int rig2icom_mode(RIG *rig, rmode_t mode, pbwidth_t width, unsigned char *md, char *pd);
+void icom2rig_mode(RIG *rig, unsigned char md, int pd, rmode_t *mode, pbwidth_t *width);
 
 #endif /* _FRAME_H */
 
