@@ -1,8 +1,8 @@
 /*
  *  Hamlib Kachina backend - main header
- *  Copyright (c) 2001 by Stephane Fillod
+ *  Copyright (c) 2001-2003 by Stephane Fillod
  *
- *		$Id: kachina.h,v 1.1 2001/08/12 22:52:44 f4cfe Exp $
+ *	$Id: kachina.h,v 1.4 2003/04/16 22:30:41 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -23,16 +23,9 @@
 #ifndef _KACHINA_H
 #define _KACHINA_H 1
 
-#if defined(__CYGWIN__)
-#  undef HAMLIB_DLL
-#  include <hamlib/rig.h>
-#  include <cal.h>
-#  define HAMLIB_DLL
-#  include <hamlib/rig_dll.h>
-#else
-#  include <hamlib/rig.h>
-#  include <cal.h>
-#endif
+#include <hamlib/rig.h>
+#include <cal.h>
+#include <tones.h>
 
 struct kachina_priv_caps {
 	cal_table_t str_cal;
@@ -43,8 +36,6 @@ int kachina_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width);
 
 extern const struct rig_caps k505dsp_caps;
 
-extern HAMLIB_EXPORT(int) init_kachina(void *be_handle);
-
 
 #endif /* _KACHINA_H */
 
