@@ -1,8 +1,8 @@
 /*
  *  Hamlib Kenwood backend - main file
- *  Copyright (c) 2000,2001 by Stephane Fillod
+ *  Copyright (c) 2000-2003 by Stephane Fillod and others
  *
- *		$Id: kenwood.c,v 1.15 2001/08/08 21:32:25 f4cfe Exp $
+ *	$Id: kenwood.c,v 1.61 2003/04/16 22:30:41 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -25,33 +25,17 @@
 #endif
 
 #include <stdlib.h>
-#include <stdio.h>   /* Standard input/output definitions */
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
-#include <fcntl.h>   /* File control definitions */
-#include <errno.h>   /* Error number definitions */
-#include <termios.h> /* POSIX terminal control definitions */
-#include <sys/ioctl.h>
 #include <math.h>
 
-#if defined(__CYGWIN__)
-#  undef HAMLIB_DLL
-#  include <hamlib/rig.h>
-#  include <serial.h>
-#  include <misc.h>
-#  define HAMLIB_DLL
-#  include <hamlib/rig_dll.h>
-#else
-#  include <hamlib/rig.h>
-#  include <serial.h>
-#  include <misc.h>
-#endif
+#include "hamlib/rig.h"
+#include "serial.h"
+#include "misc.h"
+#include "register.h"
 
 #include "kenwood.h"
 
-
-#define EOM ';'
-
 /*
  * modes in use by the "MD" command
  */
@@ -70,24 +54,46 @@ struct kenwood_id {
 	int id;
 };
 
+struct kenwood_id_string {
+	rig_model_t model;
+	const char *id;
+};
+
+
 #define UNKNOWN_ID -1
 
 /*
  * Identification number as returned by "ID;"
  * Please, if the model number of your rig is listed as UNKNOWN_ID,
- * send the value to <f4cfe@users.sourceforge.net> for inclusion. Thanks --SF
+ * send the value to <fillods@users.sourceforge.net> for inclusion. Thanks --SF
  *
  * TODO: sort this list with most frequent rigs first.
  */
 static const struct kenwood_id kenwood_id_list[] = {
+	{ RIG_MODEL_TS940, 1 },
+	{ RIG_MODEL_TS811, 2 },
+	{ RIG_MODEL_TS711, 3 },
+	{ RIG_MODEL_TS440, 4 },
 	{ RIG_MODEL_R5000, 5 },
+	{ RIG_MODEL_TS790, 7 },
+	{ RIG_MODEL_TS450S, 10 },
+	{ RIG_MODEL_TS690S, 11 },
 	{ RIG_MODEL_TS870S, 15 },
-	{ RIG_MODEL_TS570D, 17 },
+	{ RIG_MODEL_TS570D, 17 },	/* Elecraft K2 also returns 17 */
 	{ RIG_MODEL_TS570S, 18 },
 	{ RIG_MODEL_TS2000, 19 },
 	{ RIG_MODEL_NONE, UNKNOWN_ID },	/* end marker */
 };
 
+static const struct kenwood_id_string kenwood_id_string_list[] = {
+	{ RIG_MODEL_THD7A,  "TH-D7" },
+	{ RIG_MODEL_THD7AG, "TH-D7G" },
+	{ RIG_MODEL_THF6A, "TH-F6" },
+	{ RIG_MODEL_THF7E, "TH-F7" },
+	{ RIG_MODEL_NONE, NULL },	/* end marker */
+};
+
+
 /*
  * 38 CTCSS sub-audible tones
   */
@@ -99,56 +105,121 @@ const int kenwood38_ctcss_list[] = {
 	0,
 };
 
-/*
- * port_transaction
- * called by kenwood_transaction and probe_kenwood
- * We assume that port!=NULL, data!=NULL, data_len!=NULL
- * Otherwise, you'll get a nice seg fault. You've been warned!
- * return value: RIG_OK if everything's fine, negative value otherwise
- * TODO: error case handling, error codes from the rig, ..
- */
-static int port_transaction(port_t *port, const char *cmd, int cmd_len, char *data, int *data_len)
-{
-	int i, retval;
+#define cmd_trm(rig) ((struct kenwood_priv_caps *)(rig)->caps->priv)->cmdtrm
 
-	retval = write_block(port, cmd, cmd_len);
-	if (retval != RIG_OK)
-		return retval;
-
-	/*
-	 * buffered read are quite helpful here!
-	 * However, an automate with a state model would be more efficient..
-	 */
-	i = 0;
-	do {
-		retval = fread_block(port, data+i, 1);
-		if (retval == 0)
-				continue;	/* huh!? */
-		if (retval < 0)
-				return retval;
-	} while (data[i++] != EOM);
-
-	*data_len = i;
-
-	return RIG_OK;
-}
-
-/*
+/**
  * kenwood_transaction
- * We assume that rig!=NULL, rig->state!= NULL, data!=NULL, data_len!=NULL
- * Otherwise, you'll get a nice seg fault. You've been warned!
- * return value: RIG_OK if everything's fine, negative value otherwise
- * TODO: error case handling, error codes from the rig, ..
+ * Assumes rig!=NULL rig->state!=NULL rig->caps!=NULL
  *
- * Note: right now, kenwood_transaction is a simple call to port_transaction.
- * kenwood_transaction could have been written to accept a port_t instead
- * of RIG, but who knows if we will need some priv stuff in the future?
+ * cmdstr - Command to be sent to the rig. Cmdstr can also be NULL, indicating
+ *          that only a reply is needed (nothing will be send).
+ * data - Buffer for reply string.  Can be NULL, indicating that no reply is
+ *        is needed and will return with RIG_OK after command was sent.
+ * datasize - in: Size of buffer. It is the caller's responsibily to provide
+ *            a large enough buffer for all possible replies for a command.
+ *            out: location where to store number of bytes read.
+ *
+ * returns:
+ *   RIG_OK  -  if no error occured.
+ *   RIG_EIO  -  if an I/O error occured while sending/receiving data.
+ *   RIG_ETIMEOUT  -  if timeout expires without any characters received.
+ *   RIG_REJECTED  -  if a negative acknowledge was received or command not
+ *                    recognized by rig.
  */
-int kenwood_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *data_len)
+int
+kenwood_transaction (RIG *rig, const char *cmdstr, int cmd_len,
+				char *data, size_t *datasize)
 {
-	return port_transaction(&rig->state.rigport, cmd, cmd_len, data, data_len);
+    struct rig_state *rs;
+    int retval;
+    const char *cmdtrm = EOM_KEN;  /* Default Command/Reply termination char */
+    int retry_read = 0;
+#define MAX_RETRY_READ  32
+
+    rs = &rig->state;
+    rs->hold_decode = 1;
+
+	serial_flush(&rs->rigport);
+
+    cmdtrm = cmd_trm(rig);
+
+    if (cmdstr) {
+        retval = write_block(&rs->rigport, cmdstr, strlen(cmdstr));
+        if (retval != RIG_OK)
+            goto transaction_quit;
+#ifdef TH_ADD_CMDTRM
+        retval = write_block(&rs->rigport, cmdtrm, strlen(cmdtrm));
+        if (retval != RIG_OK)
+            goto transaction_quit;
+#endif
+    }
+
+    if (data == NULL || *datasize <= 0) {
+        rig->state.hold_decode = 0;
+        return RIG_OK;  /* don't want a reply */
+    }
+
+transaction_read:
+    retval = read_string(&rs->rigport, data, *datasize, cmdtrm, strlen(cmdtrm));
+    if (retval < 0)
+        goto transaction_quit;
+	else
+			*datasize = retval;
+
+    /* Check that command termination is correct */
+    if (!strchr(cmdtrm, data[strlen(data)])) {
+        if (retry_read++ < MAX_RETRY_READ)
+            goto transaction_read;
+        rig_debug(RIG_DEBUG_ERR, "%s: Command is not correctly terminated '%s'\n", __FUNCTION__, data);
+        retval = -RIG_EPROTO;
+        goto transaction_quit;
+    }
+
+    /* Command recognised by rig but invalid data entered. */
+    if (strlen(data) == 2 && data[0] == 'N') {
+        rig_debug(RIG_DEBUG_ERR, "%s: NegAck for '%s'\n", __FUNCTION__, cmdstr);
+        retval = -RIG_ERJCTED;
+        goto transaction_quit;
+    }
+
+    /* Command not understood by rig */
+    if (strlen(data) == 2 && data[0] == '?') {
+        rig_debug(RIG_DEBUG_ERR, "%s: Unknown command '%s'\n", __FUNCTION__, cmdstr);
+        retval = -RIG_ERJCTED;
+        goto transaction_quit;
+    }
+
+#define CONFIG_STRIP_CMDTRM 1
+#ifdef CONFIG_STRIP_CMDTRM
+	if (strlen(data) > 0)
+    	data[strlen(data)-1] = '\0';  /* not very elegant, but should work. */
+	else
+    	data[0] = '\0';
+#endif
+    /*
+     * Check that received the correct reply. The first two characters
+     * should be the same as command.
+     */
+    if (cmdstr && (data[0] != cmdstr[0] || data[1] != cmdstr[1])) {
+         /*
+          * TODO: When RIG_TRN is enabled, we can pass the string
+          *       to the decoder for callback. That way we don't ignore
+          *       any commands.
+          */
+        if (retry_read++ < MAX_RETRY_READ)
+            goto transaction_read;
+        rig_debug(RIG_DEBUG_ERR, "%s: Unexpected reply '%s'\n", __FUNCTION__, data);
+        retval =  -RIG_EPROTO;
+        goto transaction_quit;
+    }
+
+    retval = RIG_OK;
+transaction_quit:
+    rs->hold_decode = 0;
+    return retval;
 }
 
+
 /*
  * kenwood_set_vfo
  * Assumes rig!=NULL
@@ -156,7 +227,7 @@ int kenwood_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int
 int kenwood_set_vfo(RIG *rig, vfo_t vfo)
 {
 		unsigned char cmdbuf[16], ackbuf[16];
-		int ack_len, retval;
+		int cmd_len, ack_len, retval;
 		char vfo_function;
 
 			/*
@@ -174,34 +245,36 @@ int kenwood_set_vfo(RIG *rig, vfo_t vfo)
 								vfo);
 			return -RIG_EINVAL;
 		}
-		cmdbuf[0] = 'F';
-		cmdbuf[1] = 'R';
-		cmdbuf[2] = vfo_function;
-		cmdbuf[3] = EOM;
+
+		cmd_len = sprintf(cmdbuf, "FR%c%s", vfo_function, cmd_trm(rig));
 
 		/* set RX VFO */
-		retval = kenwood_transaction (rig, cmdbuf, 4, ackbuf, &ack_len);
+		ack_len = 0;
+		retval = kenwood_transaction (rig, cmdbuf, cmd_len, ackbuf, &ack_len);
 		if (retval != RIG_OK)
 			return retval;
 
 		/* set TX VFO */
 		cmdbuf[1] = 'T';
+		ack_len = 0;
 		retval = kenwood_transaction (rig, cmdbuf, 4, ackbuf, &ack_len);
 
 		return retval;
 }
 
+
 /*
  * kenwood_get_vfo
  * Assumes rig!=NULL, !vfo
  */
 int kenwood_get_vfo(RIG *rig, vfo_t *vfo)
 {
-		unsigned char vfobuf[16];
+		unsigned char vfobuf[50];
 		int vfo_len, retval;
 
 
 		/* query RX VFO */
+		vfo_len = 50;
 		retval = kenwood_transaction (rig, "FR;", 3, vfobuf, &vfo_len);
 		if (retval != RIG_OK)
 			return retval;
@@ -225,6 +298,77 @@ int kenwood_get_vfo(RIG *rig, vfo_t *vfo)
 		return RIG_OK;
 }
 
+/*
+ * kenwood_old_set_vfo
+ * Assumes rig!=NULL
+ * for TS-940, TS-811, TS-711 and TS-440
+ */
+int kenwood_old_set_vfo(RIG *rig, vfo_t vfo)
+{
+		unsigned char cmdbuf[16], ackbuf[16];
+		int cmd_len, ack_len, retval;
+		char vfo_function;
+
+			/*
+			 * FIXME: vfo==RIG_VFO_CURR
+			 */
+
+		switch (vfo) {
+		case RIG_VFO_VFO:
+		case RIG_VFO_A: vfo_function = '0'; break;
+		case RIG_VFO_B: vfo_function = '1'; break;
+		case RIG_VFO_MEM: vfo_function = '2'; break;
+		/* TODO : case RIG_VFO_C: */ 
+		default: 
+			rig_debug(RIG_DEBUG_ERR,"kenwood_set_vfo: unsupported VFO %d\n",
+								vfo);
+			return -RIG_EINVAL;
+		}
+
+		cmd_len = sprintf(cmdbuf, "FN%c%s", vfo_function, cmd_trm(rig));
+
+		ack_len = 0;
+		retval = kenwood_transaction (rig, cmdbuf, cmd_len, ackbuf, &ack_len);
+		return retval;
+}
+
+/*
+ * kenwood_old_get_vfo
+ * Assumes rig!=NULL, !vfo
+ * for TS-940, TS-811, TS-711 and TS-440
+ */
+int kenwood_old_get_vfo(RIG *rig, vfo_t *vfo)
+{
+		unsigned char vfobuf[50];
+		int vfo_len, retval;
+
+
+		/* query RX VFO */
+		vfo_len = 50;
+		retval = kenwood_transaction (rig, "FN;", 3, vfobuf, &vfo_len);
+		if (retval != RIG_OK)
+			return retval;
+
+		if (vfo_len != 4 || vfobuf[1] != 'N') {
+			rig_debug(RIG_DEBUG_ERR,"%s: unexpected answer %s, "
+				"len=%d\n", __FUNCTION__, vfobuf, vfo_len);
+			return -RIG_ERJCTED;
+		}
+
+		/* TODO: replace 0,1,2,.. constants by defines */
+		switch (vfobuf[2]) {
+		case '0': *vfo = RIG_VFO_A; break;
+		case '1': *vfo = RIG_VFO_B; break;
+		case '2': *vfo = RIG_VFO_MEM; break;
+		default: 
+			rig_debug(RIG_DEBUG_ERR,"%s: unsupported VFO %c\n",
+						__FUNCTION__, vfobuf[2]);
+			return -RIG_EPROTO;
+		}
+		return RIG_OK;
+}
+
+
 /*
  * kenwood_set_freq
  * Assumes rig!=NULL
@@ -255,6 +399,7 @@ int kenwood_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 		}
 		freq_len = sprintf(freqbuf,"F%c%011Ld;", vfo_letter, freq);
 
+		ack_len = 0;
 		retval = kenwood_transaction (rig, freqbuf, freq_len, ackbuf, &ack_len);
 
 		return retval;
@@ -266,9 +411,9 @@ int kenwood_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
  */
 int kenwood_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 {
-		unsigned char freqbuf[16];
+		unsigned char freqbuf[50];
 		unsigned char cmdbuf[4];
-		int freq_len, retval;
+		int cmd_len, freq_len, retval;
 		char vfo_letter;
 
 			/*
@@ -285,23 +430,95 @@ int kenwood_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 		case RIG_VFO_B: vfo_letter = 'B'; break;
 		case RIG_VFO_C: vfo_letter = 'C'; break;
 		default: 
-			rig_debug(RIG_DEBUG_ERR,"kenwood_set_freq: unsupported VFO %d\n",
+			rig_debug(RIG_DEBUG_ERR,"kenwood_get_freq: unsupported VFO %d\n",
 								vfo);
 			return -RIG_EINVAL;
 		}
-		cmdbuf[0] = 'F';
-		cmdbuf[1] = vfo_letter;
-		cmdbuf[2] = EOM;
 
-		retval = kenwood_transaction (rig, cmdbuf, 3, freqbuf, &freq_len);
+		cmd_len = sprintf(cmdbuf, "F%c%s", vfo_letter, cmd_trm(rig));
+
+		freq_len = 50;
+		retval = kenwood_transaction (rig, cmdbuf, cmd_len, freqbuf, &freq_len);
 		if (retval != RIG_OK)
 			return retval;
 
-		sscanf(freqbuf,"%Ld", freq);
+		if (freq_len != 14 || freqbuf[0] != 'F') {
+			rig_debug(RIG_DEBUG_ERR,"kenwood_get_freq: unexpected answer %s, "
+							"len=%d\n", freqbuf, freq_len);
+			return -RIG_ERJCTED;
+		}
+
+		sscanf(freqbuf+2, "%lld", freq);
 
 		return RIG_OK;
 }
 
+int kenwood_get_rit(RIG *rig, vfo_t vfo, shortfreq_t * rit)
+{
+		unsigned char infobuf[50];
+		int info_len, retval;
+
+                info_len = 50;
+		retval = kenwood_transaction (rig, "IF;", 3, infobuf, &info_len);
+		if (retval != RIG_OK)
+			return retval;
+
+		if (info_len != 38 || infobuf[1] != 'F') {
+		rig_debug(RIG_DEBUG_ERR,"kenwood_get_rit: wrong answer len=%d\n",
+						info_len);
+		return -RIG_ERJCTED;
+		}
+
+		infobuf[23] = '\0';
+		*rit = atoi(&infobuf[18]);
+
+		return RIG_OK;
+}
+
+/*
+ * rit can only move up/down by 10 Hz, so we use a loop...
+ */
+int kenwood_set_rit(RIG * rig, vfo_t vfo, shortfreq_t rit)
+{
+        unsigned char buf[50], infobuf[50], c;
+        int retval, info_len, len, i;
+
+	
+	info_len = 0;
+        if (rit == 0)
+        return kenwood_transaction(rig, "RC;", 3, infobuf, &info_len);
+
+        if (rit > 0)
+                c = 'U';
+        else
+                c = 'D';
+        len = sprintf(buf, "R%c;", c);
+
+	info_len = 0;
+	retval = kenwood_transaction(rig, "RC;", 3, infobuf, &info_len);
+	for (i = 0; i < abs(rint(rit/10)); i++)
+	{
+		info_len = 0;
+		retval = kenwood_transaction(rig, buf, len, infobuf, &info_len);
+	}
+
+	return RIG_OK;
+}
+
+/* 
+ * rit and xit are the same
+ */
+int kenwood_get_xit(RIG * rig, vfo_t vfo, shortfreq_t * rit)
+{
+        return kenwood_get_rit(rig, vfo, rit);
+}
+
+int kenwood_set_xit(RIG * rig, vfo_t vfo, shortfreq_t rit)
+{
+        return kenwood_set_rit(rig, vfo, rit);
+}
+
+
 /*
  * kenwood_set_mode
  * Assumes rig!=NULL
@@ -313,11 +530,13 @@ int kenwood_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 
 		switch (mode) {
 			case RIG_MODE_CW:       kmode = MD_CW; break;
+			case RIG_MODE_CWR:      kmode = MD_CWR; break;
 			case RIG_MODE_USB:      kmode = MD_USB; break;
 			case RIG_MODE_LSB:      kmode = MD_LSB; break;
 			case RIG_MODE_FM:       kmode = MD_FM; break;
 			case RIG_MODE_AM:       kmode = MD_AM; break;
 			case RIG_MODE_RTTY:     kmode = MD_FSK; break;
+			case RIG_MODE_RTTYR:    kmode = MD_FSKR; break;
 			default:
 				rig_debug(RIG_DEBUG_ERR,"kenwood_set_mode: "
 								"unsupported mode %d\n", mode);
@@ -325,6 +544,7 @@ int kenwood_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 		}
 
 		mdbuf_len = sprintf(mdbuf, "MD%c;", kmode);
+		ack_len = 0;
 		retval = kenwood_transaction (rig, mdbuf, mdbuf_len, ackbuf, &ack_len);
 
 		return retval;
@@ -336,41 +556,86 @@ int kenwood_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
  */
 int kenwood_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 {
-		unsigned char ackbuf[16];
-		int ack_len, retval;
+		unsigned char modebuf[50];
+		int mode_len, retval;
 
 
-		retval = kenwood_transaction (rig, "MD;", 2, ackbuf, &ack_len);
+		mode_len = 50;
+		retval = kenwood_transaction (rig, "MD;", 3, modebuf, &mode_len);
 		if (retval != RIG_OK)
 			return retval;
 
-		if (ack_len != 4) {
-				rig_debug(RIG_DEBUG_ERR,"kenwood_get_mode: ack NG, len=%d\n",
-								ack_len);
+		if (mode_len != 4 || modebuf[1] != 'D') {
+				rig_debug(RIG_DEBUG_ERR,"kenwood_get_mode: unexpected answer, len=%d\n",
+								mode_len);
 				return -RIG_ERJCTED;
 		}
 
-		*width = RIG_PASSBAND_NORMAL;	/* FIXME */
-		switch (ackbuf[0]) {
-			case MD_CW:		*mode = RIG_MODE_CW; break;
+		switch (modebuf[2]) {
+			case MD_CW:	*mode = RIG_MODE_CW; break;
+			case MD_CWR:	*mode = RIG_MODE_CWR; break;
 			case MD_USB:	*mode = RIG_MODE_USB; break;
 			case MD_LSB:	*mode = RIG_MODE_LSB; break;
-			case MD_FM:		*mode = RIG_MODE_FM; break;
-			case MD_AM:		*mode = RIG_MODE_AM; break;
-			case MD_FSK:		*mode = RIG_MODE_RTTY; break;
-#ifdef RIG_MODE_CWR
-			case MD_CWR:		*mode = RIG_MODE_CWR; break;
-#endif
-#ifdef RIG_MODE_RTTYR
-			case MD_FSKR:		*mode = RIG_MODE_RTTY; break;
-#endif
-			case MD_NONE:		*mode = RIG_MODE_NONE; break;
+			case MD_FM:	*mode = RIG_MODE_FM; break;
+			case MD_AM:	*mode = RIG_MODE_AM; break;
+			case MD_FSK:	*mode = RIG_MODE_RTTY; break;
+			case MD_FSKR:	*mode = RIG_MODE_RTTYR; break;
+			case MD_NONE:	*mode = RIG_MODE_NONE; break;
 			default:
 				rig_debug(RIG_DEBUG_ERR,"kenwood_get_mode: "
-								"unsupported mode %d\n", ackbuf[0]);
+								"unsupported mode '%c'\n", modebuf[2]);
 				return -RIG_EINVAL;
 		}
 
+                *width = rig_passband_normal(rig, *mode);
+
+		return RIG_OK;
+}
+
+int kenwood_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
+{
+		unsigned char levelbuf[16], ackbuf[16];
+		int level_len, ack_len, retval;
+		int kenwood_val;
+
+		if (RIG_LEVEL_IS_FLOAT(level))
+				kenwood_val = val.f * 255;
+		else
+				kenwood_val = val.i;
+
+		switch (level) {
+		case RIG_LEVEL_RFPOWER:
+		level_len = sprintf(levelbuf, "PC%03d;", kenwood_val);
+		break;
+
+		case RIG_LEVEL_AF:
+		level_len = sprintf(levelbuf, "AG%03d;", kenwood_val);
+		break;
+
+		case RIG_LEVEL_RF:
+		level_len = sprintf(levelbuf, "RG%03d;", kenwood_val);
+		break;
+
+		case RIG_LEVEL_SQL:
+		level_len = sprintf(levelbuf, "SQ%03d;", kenwood_val);
+		break;
+
+		case RIG_LEVEL_AGC:
+		if (kenwood_val > 3) kenwood_val = 3; /* 0.. 255 */
+		level_len = sprintf(levelbuf, "GT%03d;", 84*kenwood_val);
+		break;
+
+		default:
+			rig_debug(RIG_DEBUG_ERR,"Unsupported set_level %d", level);
+			return -RIG_EINVAL;
+		}
+
+		ack_len = 0;
+		retval = kenwood_transaction (rig, levelbuf, level_len, ackbuf, &ack_len);
+
+		if (retval != RIG_OK)
+				return retval;
+
 		return RIG_OK;
 }
 
@@ -379,15 +644,16 @@ int kenwood_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
  */
 static int get_kenwood_level(RIG *rig, const char *cmd, int cmd_len, float *f)
 {
-	unsigned char lvlbuf[16];
+	unsigned char lvlbuf[50];
 	int lvl_len, retval;
 	int lvl;
 
+	lvl_len = 50;
 	retval = kenwood_transaction (rig, cmd, cmd_len, lvlbuf, &lvl_len);
 	if (retval != RIG_OK)
 		return retval;
 
-	if (lvl_len != 4) {
+	if (lvl_len != 6) {
 		rig_debug(RIG_DEBUG_ERR,"kenwood_get_level: wrong answer len=%d\n",
 				lvl_len);
 		return -RIG_ERJCTED;
@@ -409,36 +675,29 @@ static int get_kenwood_level(RIG *rig, const char *cmd, int cmd_len, float *f)
  */
 int kenwood_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 {
-		unsigned char lvlbuf[16];
+		unsigned char lvlbuf[50];
 		int lvl_len, retval;
 		int lvl;
-		int i;
+		int i, ret, agclevel;
 
-		/* Optimize:
-		 *   sort the switch cases with the most frequent first
-		 */
+		lvl_len = 50;
 		switch (level) {
 		case RIG_LEVEL_STRENGTH:
 			retval = kenwood_transaction (rig, "SM;", 3, lvlbuf, &lvl_len);
 			if (retval != RIG_OK)
 				return retval;
 
-			if (lvl_len != 4) {
+			if (lvl_len != 7 || lvlbuf[1] != 'M') {
 				rig_debug(RIG_DEBUG_ERR,"kenwood_get_level: "
 								"wrong answer len=%d\n", lvl_len);
 				return -RIG_ERJCTED;
 			}
 
-			/* FIXME: should be in dB ! */
+			/* Frontend expects:  -54 = S0, 0 = S9  */
 			sscanf(lvlbuf+2, "%d", &val->i);
+			val->i = (val->i * 4) - 54;
 			break;
 
-		case RIG_LEVEL_SQLSTAT:
-			return -RIG_ENIMPL;	/* get_dcd ? */
-
-		case RIG_LEVEL_PREAMP:
-			return -RIG_ENIMPL;
-
 		case RIG_LEVEL_ATT:
 			retval = kenwood_transaction (rig, "RA;", 3, lvlbuf, &lvl_len);
 			if (retval != RIG_OK)
@@ -465,6 +724,9 @@ int kenwood_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 					val->i = rig->state.attenuator[i-1];
 			}
 			break;
+		case RIG_LEVEL_RFPOWER:
+			return get_kenwood_level(rig, "PC;", 3, &val->f);
+
 		case RIG_LEVEL_AF:
 			return get_kenwood_level(rig, "AG;", 3, &val->f);
 
@@ -477,6 +739,17 @@ int kenwood_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		case RIG_LEVEL_MICGAIN:
 			return get_kenwood_level(rig, "MG;", 3, &val->f);
 
+		case RIG_LEVEL_AGC:
+			ret = get_kenwood_level(rig, "GT;", 3, &val->f);
+			agclevel = 255 * val->f;
+			if (agclevel == 0) val->i = 0;
+			else if (agclevel < 85) val->i = 1;
+			else if (agclevel < 170) val->i = 2;
+			else if (agclevel <= 255) val->i = 3;
+			return ret;
+
+		case RIG_LEVEL_SQLSTAT:
+		case RIG_LEVEL_PREAMP:
 		case RIG_LEVEL_IF:
 		case RIG_LEVEL_APF:
 		case RIG_LEVEL_NR:
@@ -486,7 +759,6 @@ int kenwood_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		case RIG_LEVEL_KEYSPD:
 		case RIG_LEVEL_NOTCHF:
 		case RIG_LEVEL_COMP:
-		case RIG_LEVEL_AGC:
 		case RIG_LEVEL_BKINDL:
 		case RIG_LEVEL_BALANCE:
 			return -RIG_ENIMPL;
@@ -499,15 +771,76 @@ int kenwood_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		return RIG_OK;
 }
 
+
+int kenwood_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
+{
+		unsigned char fctbuf[16], ackbuf[16];
+		int fct_len, ack_len;
+
+		/* Optimize:
+		 *   sort the switch cases with the most frequent first
+		 */
+		ack_len = 0;
+		switch (func) {
+		case RIG_FUNC_NB:
+		fct_len = sprintf(fctbuf,"NB%c;", status==RIG_FUNC_NB?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		case RIG_FUNC_ABM:
+		fct_len = sprintf(fctbuf,"AM%c;", status==RIG_FUNC_ABM?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		case RIG_FUNC_COMP:
+		fct_len = sprintf(fctbuf,"PR%c;", status==RIG_FUNC_COMP?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		case RIG_FUNC_TONE:
+		fct_len = sprintf(fctbuf,"TO%c;", status==RIG_FUNC_TONE?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		case RIG_FUNC_TSQL:
+		fct_len = sprintf(fctbuf,"CT%c;", status==RIG_FUNC_TSQL?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		case RIG_FUNC_VOX:
+		fct_len = sprintf(fctbuf,"VX%c;", status==RIG_FUNC_VOX?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		case RIG_FUNC_NR:
+		fct_len = sprintf(fctbuf,"NR%c;", status==RIG_FUNC_NR?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		case RIG_FUNC_BC:
+		fct_len = sprintf(fctbuf,"BC%c;", status==RIG_FUNC_BC?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		case RIG_FUNC_ANF:
+		fct_len = sprintf(fctbuf,"NT%c;", status==RIG_FUNC_ANF?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		case RIG_FUNC_LOCK:
+		fct_len = sprintf(fctbuf,"LK%c;", status==RIG_FUNC_LOCK?'0':'1');
+		return kenwood_transaction (rig, fctbuf, fct_len, ackbuf, &ack_len);
+
+		default:
+			rig_debug(RIG_DEBUG_ERR,"Unsupported set_func %#x", func);
+			return -RIG_EINVAL;
+		}
+
+		return RIG_OK;
+}
+
+
 /* 
  * assumes status!=NULL
  * works for any 'format 1' command
  */
 static int get_kenwood_func(RIG *rig, const char *cmd, int cmd_len, int *status)
 {
-	unsigned char fctbuf[16];
+	unsigned char fctbuf[50];
 	int fct_len, retval;
 
+	fct_len = 50;
 	retval = kenwood_transaction (rig, cmd, cmd_len, fctbuf, &fct_len);
 	if (retval != RIG_OK)
 		return retval;
@@ -530,9 +863,10 @@ static int get_kenwood_func(RIG *rig, const char *cmd, int cmd_len, int *status)
  */
 int kenwood_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
 {
-		unsigned char fctbuf[16];
+		unsigned char fctbuf[50];
 		int fct_len, retval;
 
+		fct_len = 50;
 		/* Optimize:
 		 *   sort the switch cases with the most frequent first
 		 */
@@ -595,7 +929,7 @@ int kenwood_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
  * Assumes rig!=NULL, rig->caps->ctcss_list != NULL
  *
  * Warning! This is untested stuff! May work at least on TS-870S
- * 	Please owners report to me <f4cfe@users.sourceforge.net>, thanks. --SF
+ * 	Please owners report to me <fillods@users.sourceforge.net>, thanks. --SF
 
  * TODO: TS-2000 uses CN/CT
  */
@@ -619,6 +953,7 @@ int kenwood_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
 /* TODO: replace menu no 57 by a define */
 	tone_len = sprintf(tonebuf,"EX%03d%04d;", 57, i+1);
 	
+	ack_len = 0;
 	return kenwood_transaction (rig, tonebuf, tone_len, ackbuf, &ack_len);
 }
 
@@ -629,25 +964,25 @@ int kenwood_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
 int kenwood_get_ctcss_tone(RIG *rig, vfo_t vfo, tone_t *tone)
 {
 	const struct rig_caps *caps;
-	unsigned char tonebuf[16];
+	unsigned char tonebuf[50];
 	int tone_len, i, retval;
 	unsigned int tone_idx;
 								 
 	caps = rig->caps;
 
-	/* TODO: replace menu no 57 by a define */
-	
-	retval = kenwood_transaction (rig, "EX057;", 6, tonebuf, &tone_len);
+	tone_len = 50;
+	retval = kenwood_transaction (rig, "IF;", 3, tonebuf, &tone_len);
 	if (retval != RIG_OK)
 		return retval;
 
-	if (tone_len != 10) {
-			rig_debug(RIG_DEBUG_ERR,"kenwood_get_ctcss_tone: unexpected reply "
-							"'%s', len=%d\n", tonebuf, tone_len);
-			return -RIG_ERJCTED;
+	if (tone_len != 38 || tonebuf[1] != 'F') {
+		rig_debug(RIG_DEBUG_ERR,"kenwood_get_ctcss_tone: wrong answer len=%d\n",
+						tone_len);
+		return -RIG_ERJCTED;
 	}
 
-	sscanf(tonebuf+5, "%u", (int*)&tone_idx);
+	tonebuf[36] = '\0';
+	tone_idx = atoi(&tonebuf[34]);
 
 	if (tone_idx == 0) {
 			rig_debug(RIG_DEBUG_ERR,"kenwood_get_ctcss_tone: Unexpected CTCSS "
@@ -668,15 +1003,39 @@ int kenwood_get_ctcss_tone(RIG *rig, vfo_t vfo, tone_t *tone)
 	return RIG_OK;
 }
 
+/*
+ * kenwood_get_ptt
+ * Assumes rig!=NULL, ptt!=NULL
+ */
+int kenwood_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
+{
+		unsigned char infobuf[50];
+		int info_len, retval;
+
+		info_len = 50;
+		retval = kenwood_transaction (rig, "IF;", 3, infobuf, &info_len);
+		if (retval != RIG_OK)
+			return retval;
+
+		if (info_len != 38 || infobuf[1] != 'F') {
+		rig_debug(RIG_DEBUG_ERR,"kenwood_get_ptt: wrong answer len=%d\n",
+						info_len);
+		return -RIG_ERJCTED;
+		}
+
+		*ptt = infobuf[28] == '0' ? RIG_PTT_OFF : RIG_PTT_ON;
+
+		return RIG_OK;
+}
+
 /*
  * kenwood_set_ptt
  * Assumes rig!=NULL
- * TODO: kenwood_get_ptt reading P8 field returned by IF;
  */
 int kenwood_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 {
 		unsigned char ackbuf[16];
-		int ack_len;
+		int ack_len = 0;
 
 		return kenwood_transaction (rig, ptt==RIG_PTT_ON? "TX;":"RX;", 3, 
 						ackbuf, &ack_len);
@@ -689,9 +1048,10 @@ int kenwood_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
  */
 int kenwood_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
 {
-	unsigned char busybuf[16];
+	unsigned char busybuf[50];
 	int busy_len, retval;
 								 
+	busy_len = 50;
 	retval = kenwood_transaction (rig, "BY;", 3, busybuf, &busy_len);
 	if (retval != RIG_OK)
 		return retval;
@@ -711,10 +1071,10 @@ int kenwood_get_dcd(RIG *rig, vfo_t vfo, dcd_t *dcd)
  * kenwood_set_trn
  * Assumes rig!=NULL
  */
-int kenwood_set_trn(RIG *rig, vfo_t vfo, int trn)
+int kenwood_set_trn(RIG *rig, int trn)
 {
 		unsigned char trnbuf[16], ackbuf[16];
-		int trn_len,ack_len;
+		int trn_len, ack_len = 0;
 
 		trn_len = sprintf(trnbuf,"AI%c;", trn==RIG_TRN_RIG?'1':'0');
 
@@ -725,11 +1085,12 @@ int kenwood_set_trn(RIG *rig, vfo_t vfo, int trn)
  * kenwood_get_trn
  * Assumes rig!=NULL, trn!=NULL
  */
-int kenwood_get_trn(RIG *rig, vfo_t vfo, int *trn)
+int kenwood_get_trn(RIG *rig, int *trn)
 {
-		unsigned char trnbuf[16];
+		unsigned char trnbuf[50];
 		int trn_len, retval;
 
+		trn_len = 50;
 		retval = kenwood_transaction (rig, "AI;", 3, trnbuf, &trn_len);
 		if (retval != RIG_OK)
 			return retval;
@@ -751,7 +1112,7 @@ int kenwood_get_trn(RIG *rig, vfo_t vfo, int *trn)
 int kenwood_set_powerstat(RIG *rig, powerstat_t status)
 {
 		unsigned char pwrbuf[16], ackbuf[16];
-		int pwr_len,ack_len;
+		int pwr_len, ack_len = 0;
 
 		pwr_len = sprintf(pwrbuf,"PS%c;", status==RIG_POWER_ON?'1':'0');
 
@@ -764,8 +1125,8 @@ int kenwood_set_powerstat(RIG *rig, powerstat_t status)
  */
 int kenwood_get_powerstat(RIG *rig, powerstat_t *status)
 {
-		unsigned char pwrbuf[16];
-		int pwr_len, retval;
+		unsigned char pwrbuf[50];
+		int pwr_len = 50, retval;
 
 		retval = kenwood_transaction (rig, "PS;", 3, pwrbuf, &pwr_len);
 		if (retval != RIG_OK)
@@ -788,7 +1149,7 @@ int kenwood_get_powerstat(RIG *rig, powerstat_t *status)
 int kenwood_reset(RIG *rig, reset_t reset)
 {
 		unsigned char rstbuf[16], ackbuf[16];
-		int rst_len,ack_len;
+		int rst_len, ack_len = 0;
 		char rst;
 
 		switch(reset) {
@@ -829,6 +1190,7 @@ int kenwood_send_morse(RIG *rig, vfo_t vfo, const char *msg)
 				strncat(morsebuf, p, buff_len);
 				strcat(morsebuf, ";");
 				morse_len = 4 + buff_len;
+				ack_len = 0;
 				retval = kenwood_transaction (rig, morsebuf, morse_len, 
 								ackbuf, &ack_len);
 				if (retval != RIG_OK)
@@ -848,7 +1210,7 @@ int kenwood_send_morse(RIG *rig, vfo_t vfo, const char *msg)
 int kenwood_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 {
 		unsigned char *cmd, ackbuf[16];
-		int ack_len;
+		int ack_len = 0;
 
 		switch(op) {
 			case RIG_OP_UP: cmd="UP;"; break;
@@ -861,6 +1223,7 @@ int kenwood_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 				return -RIG_EINVAL;
 		}
 
+		ack_len = 16;
 		return kenwood_transaction (rig, cmd, 3, ackbuf, &ack_len);
 }
 
@@ -871,7 +1234,7 @@ int kenwood_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op)
 int kenwood_set_mem(RIG *rig, vfo_t vfo, int ch)
 {
 		unsigned char membuf[16], ackbuf[16];
-		int mem_len, ack_len;
+		int mem_len, ack_len = 0;
 
 		/*
 		 * "MCbmm;"
@@ -889,7 +1252,7 @@ int kenwood_set_mem(RIG *rig, vfo_t vfo, int ch)
  */
 int kenwood_get_mem(RIG *rig, vfo_t vfo, int *ch)
 {
-		unsigned char membuf[16];
+		unsigned char membuf[50];
 		int retval, mem_len;
 
 		/*
@@ -898,6 +1261,7 @@ int kenwood_get_mem(RIG *rig, vfo_t vfo, int *ch)
 		 * b can be a space
 		 */
 
+		mem_len = 50;
 		retval = kenwood_transaction (rig, "MC;", 3, membuf, &mem_len);
 		if (retval != RIG_OK)
 				return retval;
@@ -919,9 +1283,10 @@ int kenwood_get_mem(RIG *rig, vfo_t vfo, int *ch)
  */
 const char* kenwood_get_info(RIG *rig)
 {
-	unsigned char firmbuf[16];
+	unsigned char firmbuf[50];
 	int firm_len, retval;
 								 
+	firm_len = 50;
 	retval = kenwood_transaction (rig, "TY;", 3, firmbuf, &firm_len);
 	if (retval != RIG_OK)
 		return NULL;
@@ -940,53 +1305,114 @@ const char* kenwood_get_info(RIG *rig)
 	}
 }
 
+#define IDBUFSZ 16
 
 /*
- * probe_kenwood
+ * proberigs_kenwood
+ *
+ * Notes:
+ * There's only one rig possible per port.
+ *
+ * rig_model_t probeallrigs_kenwood(port_t *port, rig_probe_func_t cfunc, rig_ptr_t data)
  */
-rig_model_t probe_kenwood(port_t *port)
+DECLARE_PROBERIG_BACKEND(kenwood)
 {
-	unsigned char idbuf[16];
-	int id_len, i, k_id;
-	int retval;
+	unsigned char idbuf[IDBUFSZ];
+	int id_len=-1, i, k_id;
+	int retval=-1;
+	int rates[] = { 57600, 9600, 4800, 1200, 0 };	/* possible baud rates */
+	int rates_idx;
 
 	if (!port)
-			return RIG_MODEL_NONE;
+		return RIG_MODEL_NONE;
+
+	if (port->type.rig != RIG_PORT_SERIAL)
+		return RIG_MODEL_NONE;
 
 	port->write_delay = port->post_write_delay = 0;
-	port->timeout = 50;
+	port->parm.serial.stop_bits = 2;
 	port->retry = 1;
 
-	retval = serial_open(port);
-	if (retval != RIG_OK)
+	/*
+	 * try for all different baud rates
+	 */
+	for (rates_idx = 0; rates[rates_idx]; rates_idx++) {
+		port->parm.serial.rate = rates[rates_idx];
+		port->timeout = 2*1000/rates[rates_idx] + 50;
+	
+		retval = serial_open(port);
+		if (retval != RIG_OK)
 			return RIG_MODEL_NONE;
+	
+		retval = write_block(port, "ID;", 3);
+		id_len = read_string(port, idbuf, IDBUFSZ, EOM_KEN EOM_TH, strlen(EOM_KEN EOM_TH));
+		close(port->fd);
 
-	retval = port_transaction(port, "ID;", 3, idbuf, &id_len);
+		if (retval != RIG_OK || id_len < 0)
+			continue;
+	}
 
-	close(port->fd);
-
-	if (retval != RIG_OK)
-			return RIG_MODEL_NONE;
+	if (retval != RIG_OK || id_len < 0 || !strcmp(idbuf, "ID;"))
+		return RIG_MODEL_NONE;
 
 	/* 
 	 * reply should be something like 'IDxxx;'
 	 */
-	if (id_len != 6) {
-			idbuf[6] = '\0';
+	if (id_len != 5 || id_len != 6) {
+			idbuf[7] = '\0';
 			rig_debug(RIG_DEBUG_VERBOSE,"probe_kenwood: protocol error,"
 							" expected %d, received %d: %s\n",
 							6, id_len, idbuf);
 			return RIG_MODEL_NONE;
 	}
 
+
+	/* first, try ID string */
+	for (i=0; kenwood_id_string_list[i].model != RIG_MODEL_NONE; i++) {
+			if (!strncmp(kenwood_id_string_list[i].id, idbuf+2, 16)) {
+					rig_debug(RIG_DEBUG_VERBOSE,"probe_kenwood: "
+									"found %s\n", idbuf+2);
+					if (cfunc)
+						(*cfunc)(port, kenwood_id_string_list[i].model, data);
+					return kenwood_id_string_list[i].model;
+			}
+	}
+
+	/* then, try ID numbers */
+
 	k_id = atoi(idbuf+2);
 
+	/*
+	 * Elecraft K2 returns same ID as TS570
+	 */
+	if (k_id == 17) {
+		retval = serial_open(port);
+		if (retval != RIG_OK)
+			return RIG_MODEL_NONE;
+		retval = write_block(port, "K2;", 3);
+    		id_len = read_string(port, idbuf, IDBUFSZ, EOM_KEN EOM_TH, 2);
+		close(port->fd);
+		if (retval != RIG_OK)
+			return RIG_MODEL_NONE;
+		/* 
+		 * reply should be something like 'K2n;'
+		 */
+		if (id_len == 4 || !strcmp(idbuf, "K2")) {
+			rig_debug(RIG_DEBUG_VERBOSE,"probe_kenwood: found K2\n");
+			if (cfunc)
+				(*cfunc)(port, RIG_MODEL_K2, data);
+			return RIG_MODEL_K2;
+		}
+	}
+
 	for (i=0; kenwood_id_list[i].model != RIG_MODEL_NONE; i++) {
-			if (kenwood_id_list[i].id == k_id) {
-					rig_debug(RIG_DEBUG_VERBOSE,"probe_kenwood: "
-									"found %03d\n", k_id);
-					return kenwood_id_list[i].model;
-			}
+		if (kenwood_id_list[i].id == k_id) {
+			rig_debug(RIG_DEBUG_VERBOSE,"probe_kenwood: "
+							"found %03d\n", k_id);
+			if (cfunc)
+				(*cfunc)(port, kenwood_id_list[i].model, data);
+			return kenwood_id_list[i].model;
+		}
 	}
 	/*
 	 * not found in known table.... 
@@ -1000,17 +1426,80 @@ rig_model_t probe_kenwood(port_t *port)
 }
 
 
-/*
- * init_kenwood is called by rig_backend_load
+/* kenwood_init
+ *
+ * Basically, it sets up *priv
+ * REM: serial port is already open (rig->state.rigport.fd)
  */
-int init_kenwood(void *be_handle)
+int kenwood_init(RIG *rig)
+{
+    const struct rig_caps *caps;
+    const struct kenwood_priv_caps *priv_caps;
+    rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __FUNCTION__);
+
+    if (!rig || !rig->caps)
+        return -RIG_EINVAL;
+
+    caps = rig->caps;
+    if (!caps->priv)
+        return -RIG_ECONF;
+
+    priv_caps = (const struct kenwood_priv_caps *) caps->priv;
+
+#if 0   /* No private data for Kenwood backends */
+    priv = (struct kenwood_priv_data *)malloc(sizeof(struct kenwood_priv_data));
+    if (!priv) {
+        /* whoops! memory shortage! */
+        return -RIG_ENOMEM;
+    }
+
+    rig->state.priv = (void *)priv;
+    /* Assign default values */
+    priv->dummy = -1;  // placeholder for real entries.
+#endif
+
+    return RIG_OK;
+}
+
+/* kenwood_cleanup
+ * the serial port is closed by the frontend
+ */
+int kenwood_cleanup(RIG *rig)
+{
+    rig_debug(RIG_DEBUG_TRACE, "%s: called\n", __FUNCTION__);
+
+    if (!rig)
+        return -RIG_EINVAL;
+
+    if (rig->state.priv)
+        free(rig->state.priv);
+    rig->state.priv = NULL;
+
+    return RIG_OK;
+}
+
+/*
+ * initrigs_kenwood is called by rig_backend_load
+ */
+DECLARE_INITRIG_BACKEND(kenwood)
 {
 		rig_debug(RIG_DEBUG_VERBOSE, "kenwood: _init called\n");
 
+		rig_register(&ts950sdx_caps);
+		rig_register(&ts50s_caps);
+		rig_register(&ts440_caps);
 		rig_register(&ts450s_caps);
 		rig_register(&ts570d_caps);
 		rig_register(&ts570s_caps);
+		rig_register(&ts790_caps);
+		rig_register(&ts850_caps);
 		rig_register(&ts870s_caps);
+		rig_register(&ts930_caps);
+		rig_register(&ts2000_caps);
+		rig_register(&k2_caps);
+
+		rig_register(&thd7a_caps);
+		rig_register(&thf7e_caps);
 
 		return RIG_OK;
 }
