@@ -1,10 +1,10 @@
 /*
  *  Hamlib Interface - serial communication low-level support
- *  Copyright (c) 2000,2001 by Stephane Fillod and Frank Singleton
+ *  Copyright (c) 2000-2003 by Stephane Fillod and Frank Singleton
  *  Parts of the PTT handling are derived from soundmodem, an excellent
  *  ham packet softmodem written by Thomas Sailer, HB9JNX.
  *
- *		$Id: serial.c,v 1.18 2001/08/22 21:12:09 f4cfe Exp $
+ *	$Id: serial.c,v 1.31 2003/04/19 11:49:43 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -35,16 +35,25 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
+#ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
+#endif
+
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
 
 #ifdef HAVE_TERMIOS_H
 #include <termios.h> /* POSIX terminal control definitions */
-#endif
+#else
 #ifdef HAVE_TERMIO_H
 #include <termio.h>
 #else	/* sgtty */
+#ifdef HAVE_SGTTY_H
 #include <sgtty.h>
 #endif
+#endif
+#endif
 
 /* for CTS/RTS and DTR/DSR control under Win32 --SF */
 #ifdef HAVE_WINDOWS_H
@@ -57,7 +66,6 @@
 #include <winbase.h>
 #endif
 
-#define HAMLIB_DLL
 #include <hamlib/rig.h>
 #include "serial.h"
 #include "misc.h"
@@ -87,17 +95,7 @@
 int serial_open(port_t *rp) {
 
   int fd;				/* File descriptor for the port */
-  speed_t speed;			/* serial comm speed */
-
-#ifdef HAVE_TERMIOS_H
-  struct termios options;
-#elif defined(HAVE_TERMIO_H)
-  struct termio options;
-#elif defined(HAVE_SGTTY_H)
-  struct sgttyb sg;
-#else
-#error "No term control supported!"
-#endif
+  int err;
 
   if (!rp)
 		  return -RIG_EINVAL;
@@ -116,6 +114,45 @@ int serial_open(port_t *rp) {
     return -RIG_EIO;
   }
  
+  rp->fd = fd;
+
+  err = serial_setup(rp);
+  if (err != RIG_OK) {
+		  close(fd);
+		  return err;
+  }
+
+  rp->stream = fdopen(fd, "r+b");
+  if (rp->stream == NULL) {
+		rig_debug(RIG_DEBUG_ERR, "open_serial: fdopen failed: %s\n", 
+					strerror(errno));
+		close(fd);
+		return -RIG_EIO;		/* arg, so close! */
+  }
+
+  return RIG_OK;
+}
+
+
+int serial_setup(port_t *rp)
+{
+  int fd;
+#ifdef HAVE_TERMIOS_H
+  speed_t speed;			/* serial comm speed */
+  struct termios options;
+#elif defined(HAVE_TERMIO_H)
+  struct termio options;
+#elif defined(HAVE_SGTTY_H)
+  struct sgttyb sg;
+#else
+#error "No term control supported!"
+#endif
+
+  if (!rp)
+		  return -RIG_EINVAL;
+
+  fd = rp->fd;
+
   /*
    * Get the current options for the port...
    */
@@ -127,7 +164,11 @@ int serial_open(port_t *rp) {
 #else	/* sgtty */
   ioctl (fd, TIOCGETP, &sg);
 #endif
-  
+
+#ifdef HAVE_CFMAKERAW
+  cfmakeraw(&options); /* Set serial port to RAW mode by default. */
+#endif
+
   /*
    * Set the baud rates to requested values
    */
@@ -325,285 +366,21 @@ int serial_open(port_t *rp) {
   }
 #endif
 
-  rp->stream = fdopen(fd, "r+b");
-  if (rp->stream == NULL) {
-		rig_debug(RIG_DEBUG_ERR, "open_serial: fdopen failed: %s\n", 
-					strerror(errno));
-		close(fd);
-		return -RIG_EIO;		/* arg, so close! */
-  }
-
-  rp->fd = fd;
-
   return RIG_OK;
 }
 
 
-
-
 /*
- * Read "num" bytes from "fd" and put results into
- * an array of unsigned char pointed to by "rxbuffer"
- * 
- * Sleeps read_delay milliseconds until number of bytes to read
- * is the amount requested.
- *
- * It then reads "num" bytes into rxbuffer.
- *
- * input:
- * fd - file descriptor
- * rxbuffer - ptr to rx buffer
- * num - number of bytes to read
- * read_delay - number of msec between read attempts
- *
- * returns:
- *
- * n - numbers of bytes read
- *
+ * Flush all characters waiting in RX buffer.
  */
 
-/* please, use read_block instead, and forget about non-portable FIONREAD */
-#ifdef WANT_DEPRECATED_READSLEEP
-
-int read_sleep(int fd, unsigned char *rxbuffer, int num , int read_delay) {  
-  int bytes = 0;
-  int n = 0;
-
-  rig_debug(RIG_DEBUG_ERR,"serial:read_sleep called with num = %i \n",num);
-
-  while(1) {
-    ioctl(fd, FIONREAD, &bytes); /* get bytes in buffer */
-    rig_debug(RIG_DEBUG_TRACE,"bytes  = %i\n", bytes);
-    if (bytes == num)
-      break;
-    usleep(read_delay*1000);	/* sleep read_delay msecs */
-  }
-
-  /* this should not block now */
-  
-  n = read(fd,rxbuffer,num);	/* grab num bytes from rig */
-
-  dump_hex(rxbuffer,num);
-  return n;			/* return bytes read */
-}
-
-#endif	/* WANT_DEPRECATED_READSLEEP */
-
-/*
- * Write a block of count characters to file descriptor,
- * with a pause between each character if write_delay is > 0
- *
- * The write_delay is for Yaesu type rigs..require 5 character
- * sequence to be sent with 50-200msec between each char.
- *
- * Also, post_write_delay is for some Yaesu rigs (eg: FT747) that
- * get confused with sequential fast writes between cmd sequences.
- *
- *
- *
- * input:
- *
- * fd - file descriptor to write to
- * txbuffer - pointer to a command sequence array
- * count - count of byte to send from the txbuffer
- * write_delay - write delay in ms between 2 chars
- * post_write_delay - minimum delay between two writes
- * post_write_date - timeval of last write
- *
- * returns:
- *
- *  0 = OK
- *  <0 = NOT OK
- *
- * Actually, this function has nothing specific to serial comm,
- * it could work very well also with any file handle, like a socket.
- */
-
-int write_block(port_t *p, const char *txbuffer, size_t count)
+int serial_flush( port_t *p )
 {
-  int i;
+  tcflush(p->fd, TCIFLUSH);
 
-#ifdef WANT_NON_ACTIVE_POST_WRITE_DELAY
-  if (p->post_write_date.tv_sec != 0) {
-		  signed int date_delay;	/* in us */
-		  struct timeval tv;
-
-		  /* FIXME in Y2038 ... */
-		  gettimeofday(tv, NULL);
-		  date_delay = p->post_write_delay*1000 - 
-				  		((tv.tv_sec - p->post_write_date->tv_sec)*1000000 +
-				  		 (tv.tv_usec - p->post_write_date->tv_usec));
-		  if (date_delay > 0) {
-				/*
-				 * optional delay after last write 
-				 */
-				usleep(date_delay); 
-		  }
-		  p->post_write_date.tv_sec = 0;
-  }
-#endif
-
-  if (p->write_delay > 0) {
-  	for (i=0; i < count; i++) {
-		if (write(p->fd, txbuffer+i, 1) < 0) {
-			rig_debug(RIG_DEBUG_ERR,"write_block() failed - %s\n", 
-							strerror(errno));
-			return -RIG_EIO;
-    	}
-    	usleep(p->write_delay*1000);
-  	}
-  } else {
-		  write(p->fd, txbuffer, count);
-  }
-  
-  if (p->post_write_delay > 0) {
-#ifdef WANT_NON_ACTIVE_POST_WRITE_DELAY
-#define POST_WRITE_DELAY_TRSHLD 10
-
-	if (p->post_write_delay > POST_WRITE_DELAY_TRSHLD)
-		gettimeofday(p->post_write_date, NULL);
-	else
-#else
-    usleep(p->post_write_delay*1000); /* optional delay after last write */
-				   /* otherwise some yaesu rigs get confused */
-				   /* with sequential fast writes*/
-#endif
-  }
-  rig_debug(RIG_DEBUG_TRACE,"TX %d bytes\n",count);
-  dump_hex(txbuffer,count);
-  
   return RIG_OK;
 }
 
-/*
- * Read "num" bytes from "fd" and put results into
- * an array of unsigned char pointed to by "rxbuffer"
- * 
- * Blocks on read until timeout hits.
- *
- * It then reads "num" bytes into rxbuffer.
- *
- * Actually, this function has nothing specific to serial comm,
- * it could work very well also with any file handle, like a socket.
- */
-
-int read_block(port_t *p, char *rxbuffer, size_t count)
-{  
-  fd_set rfds;
-  struct timeval tv, tv_timeout;
-  int rd_count, total_count = 0;
-  int retval;
-
-  FD_ZERO(&rfds);
-  FD_SET(p->fd, &rfds);
-
-  /*
-   * Wait up to timeout ms.
-   */
-  tv_timeout.tv_sec = p->timeout/1000;
-  tv_timeout.tv_usec = (p->timeout%1000)*1000;
-
-  while (count > 0) {
-		tv = tv_timeout;	/* select may have updated it */
-
-		retval = select(p->fd+1, &rfds, NULL, NULL, &tv);
-		if (retval == 0) {
-			rig_debug(RIG_DEBUG_WARN, "read_block: timedout after %d chars\n",
-							total_count);
-				return -RIG_ETIMEOUT;
-		}
-		if (retval < 0) {
-			rig_debug(RIG_DEBUG_ERR,"read_block: select error after %d chars: "
-							"%s\n", total_count, strerror(errno));
-				return -RIG_EIO;
-		}
-
-		/*
-		 * grab bytes from the rig
-		 * The file descriptor must have been set up non blocking.
-		 */
-  		rd_count = read(p->fd, rxbuffer+total_count, count);
-		if (rd_count < 0) {
-				rig_debug(RIG_DEBUG_ERR, "read_block: read failed - %s\n",
-									strerror(errno));
-				return -RIG_EIO;
-		}
-		total_count += rd_count;
-		count -= rd_count;
-  }
-
-  rig_debug(RIG_DEBUG_TRACE,"RX %d bytes\n",total_count);
-  dump_hex(rxbuffer, total_count);
-  return total_count;			/* return bytes count read */
-}
-
-int fread_block(port_t *p, char *rxbuffer, size_t count)
-{  
-  fd_set rfds;
-  struct timeval tv, tv_timeout;
-  int rd_count, total_count = 0;
-  int retval;
-  int fd;
-
-  fd = fileno(p->stream);
-
-  FD_ZERO(&rfds);
-  FD_SET(fd, &rfds);
-
-  /*
-   * Wait up to timeout ms.
-   */
-  tv_timeout.tv_sec = p->timeout/1000;
-  tv_timeout.tv_usec = (p->timeout%1000)*1000;
-
-
-	/*
-	 * grab bytes from the rig
-	 * The file descriptor must have been set up non blocking.
-	 */
-  	rd_count = fread(rxbuffer, 1, count, p->stream);
-	if (rd_count < 0) {
-			rig_debug(RIG_DEBUG_ERR, "read_block: read failed - %s\n",
-								strerror(errno));
-			return -RIG_EIO;
-	}
-	total_count += rd_count;
-	count -= rd_count;
-
-  while (count > 0) {
-		tv = tv_timeout;	/* select may have updated it */
-
-		retval = select(fd+1, &rfds, NULL, NULL, &tv);
-		if (retval == 0) {
-			rig_debug(RIG_DEBUG_WARN, "fread_block: timedout after %d chars\n",
-							total_count);
-				return -RIG_ETIMEOUT;
-		}
-		if (retval < 0) {
-			rig_debug(RIG_DEBUG_ERR,"fread_block: select error after %d chars: "
-							"%s\n", total_count, strerror(errno));
-				return -RIG_EIO;
-		}
-
-		/*
-		 * grab bytes from the rig
-		 * The file descriptor must have been set up non blocking.
-		 */
-  		rd_count = fread(rxbuffer+total_count, 1, count, p->stream);
-		if (rd_count < 0) {
-				rig_debug(RIG_DEBUG_ERR, "read_block: read failed - %s\n",
-									strerror(errno));
-				return -RIG_EIO;
-		}
-		total_count += rd_count;
-		count -= rd_count;
-  }
-
-  rig_debug(RIG_DEBUG_TRACE,"RX %d bytes\n",total_count);
-  dump_hex(rxbuffer, total_count);
-  return total_count;			/* return bytes count read */
-}
-
 /*
  * ser_ptt_set and par_ptt_set
  * ser_ptt_open/ser_ptt_close & par_ptt_open/par_ptt_close
@@ -651,37 +428,39 @@ int ser_close(port_t *p)
 #endif
 }
 
+int ser_set_rts(const port_t *p, int state)
+{
+#if defined(_WIN32) || defined(__CYGWIN__)
+		/*
+		 * TODO: log error with 0x%lx GetLastError()
+		 */
+	return !EscapeCommFunction(p->handle, state ? SETRTS : CLRRTS);
+#else
+	unsigned char y = TIOCM_RTS;
+	return ioctl(p->fd, state ? TIOCMBIS : TIOCMBIC, &y);
+#endif
+}
+
+int ser_set_dtr(const port_t *p, int state)
+{
+#if defined(_WIN32) || defined(__CYGWIN__)
+	return !EscapeCommFunction(p->handle, state ? SETDTR : CLRDTR);
+#else
+	unsigned char y = TIOCM_DTR;
+	return ioctl(p->fd, state ? TIOCMBIS : TIOCMBIC, &y);
+#endif
+}
+
 /* 
  * p is supposed to be &rig->state.pttport
  */
 int ser_ptt_set(port_t *p, ptt_t pttx)
 {
 		switch(p->type.ptt) {
-#if defined(_WIN32) || defined(__CYGWIN__)
-		/*
-		 * TODO: log error with 0x%lx GetLastError()
-		 */
 		case RIG_PTT_SERIAL_RTS:
-			return !EscapeCommFunction(p->handle, pttx==RIG_PTT_ON ? 
-														SETRTS : CLRRTS);
-
+			return ser_set_rts(p, pttx==RIG_PTT_ON);
 		case RIG_PTT_SERIAL_DTR:
-			return !EscapeCommFunction(p->handle, pttx==RIG_PTT_ON ? 
-														SETDTR : CLRDTR);
-#else
-		case RIG_PTT_SERIAL_RTS:
-			{
-				unsigned char y = TIOCM_RTS;
-				return ioctl(p->fd, pttx==RIG_PTT_ON ? TIOCMBIS : TIOCMBIC, &y);
-			}
-
-		case RIG_PTT_SERIAL_DTR:
-			{
-				unsigned char y = TIOCM_DTR;
-				return ioctl(p->fd, pttx==RIG_PTT_ON ? TIOCMBIS : TIOCMBIC, &y);
-			}
-#endif
-
+			return ser_set_dtr(p, pttx==RIG_PTT_ON);
 		default:
 				rig_debug(RIG_DEBUG_ERR,"Unsupported PTT type %d\n",
 								p->type.ptt);
@@ -779,8 +558,16 @@ int par_open(port_t *p)
 		int fd;
 
 		fd = open(p->pathname, O_RDWR);
+		if (fd < 0) {
+			rig_debug(RIG_DEBUG_VERBOSE, "Opening device \"%s\": %s\n", p->pathname, strerror(errno));
+			return -RIG_EIO;
+		}
 #ifdef HAVE_LINUX_PPDEV_H
-		ioctl(fd, PPCLAIM);
+		if (ioctl(fd, PPCLAIM) < 0) {
+			rig_debug(RIG_DEBUG_VERBOSE, "Claiming device \"%s\": %s\n", p->pathname, strerror(errno));
+			close(fd);
+			return -RIG_EIO;
+		}
 #endif
 		p->fd = fd;
 		return fd;
