@@ -1,8 +1,8 @@
 /*
  *  Hamlib Tentec backend - main file
- *  Copyright (c) 2001 by Stephane Fillod
+ *  Copyright (c) 2001-2003 by Stephane Fillod
  *
- *		$Id: tentec.c,v 1.1 2001/07/14 16:42:48 f4cfe Exp $
+ *	$Id: tentec.c,v 1.8 2003/04/16 22:30:43 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -25,29 +25,15 @@
 #endif
 
 #include <stdlib.h>
-#include <stdio.h>   /* Standard input/output definitions */
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
-#include <fcntl.h>   /* File control definitions */
-#include <errno.h>   /* Error number definitions */
-#include <termios.h> /* POSIX terminal control definitions */
-#include <sys/ioctl.h>
 #include <math.h>
 
-#if defined(__CYGWIN__)
-#  undef HAMLIB_DLL
-#  include <hamlib/rig.h>
-#  include <serial.h>
-#  include <misc.h>
-#  include <cal.h>
-#  define HAMLIB_DLL
-#  include <hamlib/rig_dll.h>
-#else
-#  include <hamlib/rig.h>
-#  include <serial.h>
-#  include <misc.h>
-#  include <cal.h>
-#endif
+#include "hamlib/rig.h"
+#include "serial.h"
+#include "misc.h"
+#include "cal.h"
+#include "register.h"
 
 #include "tentec.h"
 
@@ -77,11 +63,13 @@ static int tentec_filters[] = {
  */
 int tentec_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *data_len)
 {
-	int i, count, retval;
+	int retval;
 	struct rig_state *rs;
 
 	rs = &rig->state;
 
+	serial_flush(&rs->rigport);
+
 	retval = write_block(&rs->rigport, cmd, cmd_len);
 	if (retval != RIG_OK)
 			return retval;
@@ -89,20 +77,8 @@ int tentec_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *
 	/* no data expected, TODO: flush input? */
 	if (!data || !data_len)
 			return 0;
-	/*
-	 * buffered read are quite helpful here!
-	 * However, an automate with a state model would be more efficient..
-	 */
-	i = 0;
-	do {
-		count = fread_block(&rs->rigport, data+i, 1);
-		if (count > 0)
-				i += count;
-		else if (count < 0)
-				return count;
-	} while (i < *data_len);
 
-	*data_len = i;
+	*data_len = read_string(&rs->rigport, data, *data_len, "", 0);
 
 	return RIG_OK;
 }
@@ -136,11 +112,13 @@ int tentec_init(RIG *rig)
 	priv->cwbfo = 0;
 	priv->agc = 0.5;	/* medium */
 	priv->lnvol = priv->spkvol = 0.0;	/* mute */
-	tentec_tuning_factor_calc(rig);
 	memcpy(&priv->str_cal, &priv_caps->str_cal, sizeof(cal_table_t));
 
 	rig->state.priv = (rig_ptr_t)priv;
 
+	/* tentec_tuning_factor_calc needs rig->state.priv */
+	tentec_tuning_factor_calc(rig);
+
 	return RIG_OK;
 }
 
@@ -158,6 +136,25 @@ int tentec_cleanup(RIG *rig)
 	return RIG_OK;
 }
 
+/*
+ * Tentec transceiver only open routine
+ * Restart and set program to execute.
+ */
+int tentec_trx_open(RIG *rig)
+{
+	int retval;
+
+	/*
+	 * be kind: use XX first, and do 'Dsp Program Execute' only 
+	 * in " DSP START" state.
+	 */
+	retval = tentec_transaction (rig, "P1" EOM, 3, NULL, NULL);
+	if (retval != RIG_OK)
+		return retval;
+
+	return RIG_OK;
+}
+
 
 /*
  * Tuning Factor Calculations
@@ -352,7 +349,7 @@ int tentec_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 		/* FIXME: support also separate Lineout setting 
 		 * -> need to create RIG_LEVEL_LINEOUT ?
 		 */
-		cmd_len = sprintf(cmdbuf, "C%c%c" EOM, 0, (int)rint(val.f*63));
+		cmd_len = sprintf(cmdbuf, "C%c%c" EOM, 0, (int)(val.f*63));
 		retval = write_block(&rs->rigport, cmdbuf, cmd_len);
 		if (retval == RIG_OK) 
 				priv->lnvol = priv->spkvol = val.f;
@@ -441,12 +438,13 @@ const char *tentec_get_info(RIG *rig)
 
 
 /*
- * init_tentec is called by rig_backend_load
+ * initrigs_tentec is called by rig_backend_load
  */
-int init_tentec(void *be_handle)
+DECLARE_INITRIG_BACKEND(tentec)
 {
 		rig_debug(RIG_DEBUG_VERBOSE, "tentec: _init called\n");
 
+		rig_register(&tt550_caps);
 		rig_register(&rx320_caps);
 
 		return RIG_OK;
