@@ -1,9 +1,9 @@
 /*
- * dumpcaps.c - Copyright (C) 2000,2001 Stephane Fillod
+ * dumpcaps.c - Copyright (C) 2000-2003 Stephane Fillod
  * This programs dumps the capabilities of a backend rig.
  *
  *
- *    $Id: dumpcaps.c,v 1.29 2001/09/20 21:21:14 f4cfe Exp $  
+ *    $Id: dumpcaps.c,v 1.37 2003/04/06 18:40:35 fillods Exp $  
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -31,64 +31,34 @@
 #include "misc.h"
 
 
-static char *decode_modes(rmode_t modes);
-static const char *decode_mtype(enum chan_type_e type);
+static int print_ext(RIG *rig, const struct confparams *cfp, rig_ptr_t ptr);
+static const char *decode_mtype(chan_type_t type);
 int range_sanity_check(const struct freq_range_list range_list[], int rx);
 int ts_sanity_check(const struct tuning_step_list tuning_step[]);
+static void dump_chan_caps(const channel_cap_t *chan);
 
-int main (int argc, char *argv[])
+/*
+ * the rig may be in rig_init state, but not openned
+ */
+int dumpcaps (RIG* rig)
 { 
 	const struct rig_caps *caps;
 	int status,i;
 	char freqbuf[20];
 	int backend_warnings=0;
-	int rig_model;
+	char prntbuf[256];
 
-	if (argc != 2) {
-			fprintf(stderr,"%s <rig_num>\n",argv[0]);
-			exit(1);
-	}
+	if (!rig || !rig->caps)
+		return -RIG_EINVAL;
 
-	rig_model = atoi(argv[1]);
-	
-	rig_check_backend(rig_model);
+	caps = rig->caps;
 
-	caps = rig_get_caps(rig_model);
-	if (!caps) {
-			fprintf(stderr,"Unknown rig num: %d\n", rig_model);
-			fprintf(stderr,"Please check riglist.h\n");
-			exit(2);
-	}
-
-	printf("Rig dump for model %d\n",caps->rig_model);
+	printf("Caps dump for model %d\n",caps->rig_model);
 	printf("Model name:\t%s\n",caps->model_name);
 	printf("Mfg name:\t%s\n",caps->mfg_name);
 	printf("Backend version:\t%s\n",caps->version);
 	printf("Backend copyright:\t%s\n",caps->copyright);
-	printf("Backend status:\t");
-	switch (caps->status) {
-	case RIG_STATUS_ALPHA:
-			printf("Alpha\n");
-			break;
-	case RIG_STATUS_UNTESTED:
-			printf("Untested\n");
-			break;
-	case RIG_STATUS_BETA:
-			printf("Beta\n");
-			break;
-	case RIG_STATUS_STABLE:
-			printf("Stable\n");
-			break;
-	case RIG_STATUS_BUGGY:
-			printf("Buggy\n");
-			break;
-	case RIG_STATUS_NEW:
-			printf("New\n");
-			break;
-	default:
-			printf("Unknown\n");
-			backend_warnings++;
-	}
+	printf("Backend status:\t%s\n", strstatus(caps->status));
 	printf("Rig type:\t");
 	switch (caps->rig_type & RIG_TYPE_MASK) {
 	case RIG_TYPE_TRANSCEIVER:
@@ -115,6 +85,9 @@ int main (int argc, char *argv[])
 	case RIG_TYPE_COMPUTER:
 			printf("Computer\n");
 			break;
+	case RIG_TYPE_TUNER:
+			printf("Tuner\n");
+			break;
 	case RIG_TYPE_OTHER:
 			printf("Other\n");
 			break;
@@ -206,7 +179,7 @@ int main (int argc, char *argv[])
 	printf("Has transceive: %s\n",
 					caps->transceive?"yes":"no");
 
-	printf("Announce: 0x%lx\n", caps->announces);
+	printf("Announce: 0x%x\n", caps->announces);
 	printf("Max RIT: -%ld.%ldkHz/+%ld.%ldkHz\n", 
 					caps->max_rit/1000, caps->max_rit%1000,
 					caps->max_rit/1000, caps->max_rit%1000);
@@ -232,136 +205,37 @@ int main (int argc, char *argv[])
 		printf(" none");
 	printf("\n");
 
-	printf("Get functions: ");
-	if (caps->has_get_func!=0) {
-		if (caps->has_get_func&RIG_FUNC_FAGC) printf("FAGC ");
-		if (caps->has_get_func&RIG_FUNC_NB) printf("NB ");
-		if (caps->has_get_func&RIG_FUNC_COMP) printf("COMP ");
-		if (caps->has_get_func&RIG_FUNC_VOX) printf("VOX ");
-		if (caps->has_get_func&RIG_FUNC_TONE) printf("TONE ");
-		if (caps->has_get_func&RIG_FUNC_TSQL) printf("TSQL ");
-		if (caps->has_get_func&RIG_FUNC_SBKIN) printf("SBKIN ");
-		if (caps->has_get_func&RIG_FUNC_FBKIN) printf("FBKIN ");
-		if (caps->has_get_func&RIG_FUNC_ANF) printf("ANF ");
-		if (caps->has_get_func&RIG_FUNC_NR) printf("NR ");
-		if (caps->has_get_func&RIG_FUNC_AIP) printf("AIP ");
-		if (caps->has_get_func&RIG_FUNC_APF) printf("APF ");
-		if (caps->has_get_func&RIG_FUNC_MON) printf("MON ");
-		if (caps->has_get_func&RIG_FUNC_MN) printf("MN ");
-		if (caps->has_get_func&RIG_FUNC_RNF) printf("RNF ");
-		printf("\n");
-	} else
-			printf("none\n");
+	sprintf_func(prntbuf, caps->has_get_func);
+	printf("Get functions: %s\n", prntbuf);
 
-	printf("Set functions: ");
-	if (caps->has_set_func!=0) {
-		if (caps->has_set_func&RIG_FUNC_FAGC) printf("FAGC ");
-		if (caps->has_set_func&RIG_FUNC_NB) printf("NB ");
-		if (caps->has_set_func&RIG_FUNC_COMP) printf("COMP ");
-		if (caps->has_set_func&RIG_FUNC_VOX) printf("VOX ");
-		if (caps->has_set_func&RIG_FUNC_TONE) printf("TONE ");
-		if (caps->has_set_func&RIG_FUNC_TSQL) printf("TSQL ");
-		if (caps->has_set_func&RIG_FUNC_SBKIN) printf("SBKIN ");
-		if (caps->has_set_func&RIG_FUNC_FBKIN) printf("FBKIN ");
-		if (caps->has_set_func&RIG_FUNC_ANF) printf("ANF ");
-		if (caps->has_set_func&RIG_FUNC_NR) printf("NR ");
-		if (caps->has_set_func&RIG_FUNC_AIP) printf("AIP ");
-		if (caps->has_set_func&RIG_FUNC_APF) printf("APF ");
-		if (caps->has_set_func&RIG_FUNC_MON) printf("MON ");
-		if (caps->has_set_func&RIG_FUNC_MN) printf("MN ");
-		if (caps->has_set_func&RIG_FUNC_RNF) printf("RNF ");
-		printf("\n");
-	} else
-			printf("none\n");
+	sprintf_func(prntbuf, caps->has_set_func);
+	printf("Set functions: %s\n", prntbuf);
 
-	printf("Get level: ");
-	if (caps->has_get_level!=0) {
-		if (caps->has_get_level&RIG_LEVEL_PREAMP) printf("PREAMP ");
-		if (caps->has_get_level&RIG_LEVEL_ATT) printf("ATT ");
-		if (caps->has_get_level&RIG_LEVEL_AF) printf("AF ");
-		if (caps->has_get_level&RIG_LEVEL_RF) printf("RF ");
-		if (caps->has_get_level&RIG_LEVEL_SQL) printf("SQL ");
-		if (caps->has_get_level&RIG_LEVEL_IF) printf("IF ");
-		if (caps->has_get_level&RIG_LEVEL_APF) printf("APF ");
-		if (caps->has_get_level&RIG_LEVEL_NR) printf("NR ");
-		if (caps->has_get_level&RIG_LEVEL_PBT_IN) printf("PBT_IN ");
-		if (caps->has_get_level&RIG_LEVEL_PBT_OUT) printf("PBT_OUT ");
-		if (caps->has_get_level&RIG_LEVEL_CWPITCH) printf("CWPITCH ");
-		if (caps->has_get_level&RIG_LEVEL_RFPOWER) printf("RFPOWER ");
-		if (caps->has_get_level&RIG_LEVEL_MICGAIN) printf("MICGAIN ");
-		if (caps->has_get_level&RIG_LEVEL_KEYSPD) printf("KEYSPD ");
-		if (caps->has_get_level&RIG_LEVEL_NOTCHF) printf("NOTCHF ");
-		if (caps->has_get_level&RIG_LEVEL_COMP) printf("COMP ");
-		if (caps->has_get_level&RIG_LEVEL_AGC) printf("AGC ");
-		if (caps->has_get_level&RIG_LEVEL_BKINDL) printf("BKINDL ");
-		if (caps->has_get_level&RIG_LEVEL_BALANCE) printf("BALANCE ");
+	sprintf_level(prntbuf, caps->has_get_level);
+	printf("Get level: %s\n", prntbuf);
 
-		if (caps->has_get_level&RIG_LEVEL_SWR) printf("SWR ");
-		if (caps->has_get_level&RIG_LEVEL_ALC) printf("ALC ");
-		if (caps->has_get_level&RIG_LEVEL_SQLSTAT) printf("SQLSTAT ");
-		if (caps->has_get_level&RIG_LEVEL_STRENGTH) printf("STRENGTH ");
-		printf("\n");
-	} else
-			printf("none\n");
+	sprintf_level(prntbuf, caps->has_set_level);
+	printf("Set level: %s\n", prntbuf);
+	if (caps->has_set_level&RIG_LEVEL_READONLY_LIST) {
+			printf("Warning: backend can set readonly levels!\n");
+			backend_warnings++;
+	}
+	printf("Extra levels:");
+	rig_ext_level_foreach(rig, print_ext, NULL);
+	printf("\n");
 
-	printf("Set level: ");
-	if (caps->has_set_level!=0) {
-		if (caps->has_set_level&RIG_LEVEL_PREAMP) printf("PREAMP ");
-		if (caps->has_set_level&RIG_LEVEL_ATT) printf("ATT ");
-		if (caps->has_set_level&RIG_LEVEL_AF) printf("AF ");
-		if (caps->has_set_level&RIG_LEVEL_RF) printf("RF ");
-		if (caps->has_set_level&RIG_LEVEL_SQL) printf("SQL ");
-		if (caps->has_set_level&RIG_LEVEL_IF) printf("IF ");
-		if (caps->has_set_level&RIG_LEVEL_APF) printf("APF ");
-		if (caps->has_set_level&RIG_LEVEL_NR) printf("NR ");
-		if (caps->has_set_level&RIG_LEVEL_PBT_IN) printf("PBT_IN ");
-		if (caps->has_set_level&RIG_LEVEL_PBT_OUT) printf("PBT_OUT ");
-		if (caps->has_set_level&RIG_LEVEL_CWPITCH) printf("CWPITCH ");
-		if (caps->has_set_level&RIG_LEVEL_RFPOWER) printf("RFPOWER ");
-		if (caps->has_set_level&RIG_LEVEL_MICGAIN) printf("MICGAIN ");
-		if (caps->has_set_level&RIG_LEVEL_KEYSPD) printf("KEYSPD ");
-		if (caps->has_set_level&RIG_LEVEL_NOTCHF) printf("NOTCHF ");
-		if (caps->has_set_level&RIG_LEVEL_COMP) printf("COMP ");
-		if (caps->has_set_level&RIG_LEVEL_AGC) printf("AGC ");
-		if (caps->has_set_level&RIG_LEVEL_BKINDL) printf("BKINDL ");
-		if (caps->has_set_level&RIG_LEVEL_BALANCE) printf("BALANCE ");
-
-		if (caps->has_set_level&RIG_LEVEL_SWR) printf("SWR ");
-		if (caps->has_set_level&RIG_LEVEL_ALC) printf("ALC ");
-		if (caps->has_set_level&RIG_LEVEL_SQLSTAT) printf("SQLSTAT ");
-		if (caps->has_set_level&RIG_LEVEL_STRENGTH) printf("STRENGTH ");
-		printf("\n");
-		if (caps->has_set_level&RIG_LEVEL_READONLY_LIST) {
-				printf("Warning: backend can set readonly levels!\n");
-				backend_warnings++;
-		}
-	} else
-			printf("none\n");
-
-	printf("Get parameters: ");
-	if (caps->has_get_parm!=0) {
-		if (caps->has_get_parm&RIG_PARM_ANN) printf("ANN ");
-		if (caps->has_get_parm&RIG_PARM_APO) printf("APO ");
-		if (caps->has_get_parm&RIG_PARM_BACKLIGHT) printf("BACKLIGHT ");
-		if (caps->has_get_parm&RIG_PARM_BEEP) printf("BEEP ");
-		if (caps->has_get_parm&RIG_PARM_TIME) printf("TIME ");
-		if (caps->has_get_parm&RIG_PARM_BAT) printf("BAT ");
-		printf("\n");
-	} else
-			printf("none\n");
-
-	printf("Set parameters: ");
-	if (caps->has_set_parm!=0) {
-		if (caps->has_set_parm&RIG_PARM_ANN) printf("ANN ");
-		if (caps->has_set_parm&RIG_PARM_APO) printf("APO ");
-		if (caps->has_set_parm&RIG_PARM_BACKLIGHT) printf("BACKLIGHT ");
-		if (caps->has_set_parm&RIG_PARM_BEEP) printf("BEEP ");
-		if (caps->has_set_parm&RIG_PARM_TIME) printf("TIME ");
-		if (caps->has_set_parm&RIG_PARM_BAT) printf("BAT ");
-		printf("\n");
-	} else
-			printf("none\n");
+	sprintf_parm(prntbuf, caps->has_get_parm);
+	printf("Get parameters: %s\n", prntbuf);
 
+	sprintf_parm(prntbuf, caps->has_set_parm);
+	printf("Set parameters: %s\n", prntbuf);
+	if (caps->has_set_parm&RIG_PARM_READONLY_LIST) {
+			printf("Warning: backend can set readonly parms!\n");
+			backend_warnings++;
+	}
+	printf("Extra parameters:");
+	rig_ext_parm_foreach(rig, print_ext, NULL);
+	printf("\n");
 
 #if 0
 	/* FIXME: use rig->state.vfo_list instead */
@@ -382,9 +256,11 @@ int main (int argc, char *argv[])
 
 	printf("Memories:");
 	for (i=0; i<CHANLSTSIZ && caps->chan_list[i].type; i++) {
-			printf("\n\t%d..%d:   \t%s", caps->chan_list[i].start,
-							caps->chan_list[i].end,
-							decode_mtype(caps->chan_list[i].type));
+		printf("\n\t%d..%d:   \t%s", caps->chan_list[i].start,
+						caps->chan_list[i].end,
+						decode_mtype(caps->chan_list[i].type));
+		printf("\n\t  mem caps: ");
+		dump_chan_caps(&caps->chan_list[i].mem_caps);
 	}
 	if (i == 0)
 		printf(" none");
@@ -406,10 +282,13 @@ int main (int argc, char *argv[])
 	if (status) backend_warnings++;
 
 	printf("Tuning steps:");
-	for (i=0; i<TSLSTSIZ && caps->tuning_steps[i].ts; i++) {
-			freq_sprintf(freqbuf,caps->tuning_steps[i].ts);
-			printf("\n\t%s:   \t%s", freqbuf,
-							decode_modes(caps->tuning_steps[i].modes));
+	for (i=0; i<TSLSTSIZ && !RIG_IS_TS_END(caps->tuning_steps[i]); i++) {
+		if (caps->tuning_steps[i].ts == RIG_TS_ANY)
+			strcpy(freqbuf, "ANY");
+		else
+			sprintf_freq(freqbuf,caps->tuning_steps[i].ts);
+		sprintf_mode(prntbuf,caps->tuning_steps[i].modes);
+		printf("\n\t%s:   \t%s", freqbuf, prntbuf);
 	}
 	if (i==0) {
 			printf(" none! This backend might be bogus!");
@@ -421,10 +300,13 @@ int main (int argc, char *argv[])
 	if (status) backend_warnings++;
 
 	printf("Filters:");
-	for (i=0; i<FLTLSTSIZ && caps->filters[i].modes; i++) {
-			freq_sprintf(freqbuf,caps->filters[i].width);
-			printf("\n\t%s:   \t%s", freqbuf,
-							decode_modes(caps->filters[i].modes));
+	for (i=0; i<FLTLSTSIZ && !RIG_IS_FLT_END(caps->filters[i]); i++) {
+		if (caps->filters[i].width == RIG_FLT_ANY)
+			strcpy(freqbuf, "ANY");
+		else
+			sprintf_freq(freqbuf,caps->filters[i].width);
+		sprintf_mode(prntbuf,caps->filters[i].modes);
+		printf("\n\t%s:   \t%s", freqbuf, prntbuf);
 	}
 	if (i==0) {
 			printf(" none! This backend might be bogus!");
@@ -432,10 +314,34 @@ int main (int argc, char *argv[])
 	}
 	printf("\n");
 
+        printf("Bandwidths:");
+	for (i=1; i < 1<<10; i<<=1) {
+		pbwidth_t pbnorm = rig_passband_normal(rig, i);
+
+		if (pbnorm == 0)
+			continue;
+
+		sprintf_freq(freqbuf, pbnorm);
+		printf("\n\t%s\tnormal: %s,\t", strrmode(i), freqbuf);
+
+		sprintf_freq(freqbuf, rig_passband_narrow(rig, i));
+		printf("narrow: %s,\t", freqbuf);
+
+		sprintf_freq(freqbuf, rig_passband_wide(rig, i));
+		printf("wide: %s", freqbuf);
+	}
+	printf("\n");
+
 	printf("Has priv data:\t%c\n",caps->priv!=NULL?'Y':'N');
 	/*
 	 * TODO: keep me up-to-date with API call list!
 	 */
+	printf("Has init:\t%c\n",caps->rig_init!=NULL?'Y':'N');
+	printf("Has cleanup:\t%c\n",caps->rig_cleanup!=NULL?'Y':'N');
+	printf("Has open:\t%c\n",caps->rig_open!=NULL?'Y':'N');
+	printf("Has close:\t%c\n",caps->rig_close!=NULL?'Y':'N');
+	printf("Can set conf:\t%c\n",caps->set_conf!=NULL?'Y':'N');
+	printf("Can get conf:\t%c\n",caps->get_conf!=NULL?'Y':'N');
 	printf("Can set frequency:\t%c\n",caps->set_freq!=NULL?'Y':'N');
 	printf("Can get frequency:\t%c\n",caps->get_freq!=NULL?'Y':'N');
 	printf("Can set mode:\t%c\n",caps->set_mode!=NULL?'Y':'N');
@@ -453,8 +359,8 @@ int main (int argc, char *argv[])
 	printf("Can get split freq:\t%c\n",caps->get_split_freq!=NULL?'Y':'N');
 	printf("Can set split mode:\t%c\n",caps->set_split_mode!=NULL?'Y':'N');
 	printf("Can get split mode:\t%c\n",caps->get_split_mode!=NULL?'Y':'N');
-	printf("Can set split:\t%c\n",caps->set_split!=NULL?'Y':'N');
-	printf("Can get split:\t%c\n",caps->get_split!=NULL?'Y':'N');
+	printf("Can set split vfo:\t%c\n",caps->set_split_vfo!=NULL?'Y':'N');
+	printf("Can get split vfo:\t%c\n",caps->get_split_vfo!=NULL?'Y':'N');
 	printf("Can set tuning step:\t%c\n",caps->set_ts!=NULL?'Y':'N');
 	printf("Can get tuning step:\t%c\n",caps->get_ts!=NULL?'Y':'N');
 	printf("Can set RIT:\t%c\n",caps->set_rit!=NULL?'Y':'N');
@@ -471,6 +377,9 @@ int main (int argc, char *argv[])
 	printf("Can get DCS squelch:\t%c\n",caps->get_dcs_sql!=NULL?'Y':'N');
 	printf("Can set power stat:\t%c\n",caps->set_powerstat!=NULL?'Y':'N');
 	printf("Can get power stat:\t%c\n",caps->get_powerstat!=NULL?'Y':'N');
+	printf("Can reset:\t%c\n",caps->reset!=NULL?'Y':'N');
+	printf("Can get ant:\t%c\n",caps->get_ant!=NULL?'Y':'N');
+	printf("Can set ant:\t%c\n",caps->set_ant!=NULL?'Y':'N');
 	printf("Can set transceive:\t%c\n",caps->set_trn!=NULL?'Y':'N');
 	printf("Can get transceive:\t%c\n",caps->get_trn!=NULL?'Y':'N');
 	printf("Can set func:\t%c\n",caps->set_func!=NULL?'Y':'N');
@@ -483,15 +392,13 @@ int main (int argc, char *argv[])
 	printf("Can recv DTMF:\t%c\n",caps->recv_dtmf!=NULL?'Y':'N');
 	printf("Can send Morse:\t%c\n",caps->send_morse!=NULL?'Y':'N');
 	printf("Can decode events:\t%c\n",caps->decode_event!=NULL?'Y':'N');
+	printf("Can set bank:\t%c\n",caps->set_bank!=NULL?'Y':'N');
 	printf("Can set mem:\t%c\n",caps->set_mem!=NULL?'Y':'N');
 	printf("Can get mem:\t%c\n",caps->get_mem!=NULL?'Y':'N');
 	printf("Can set channel:\t%c\n",caps->set_channel!=NULL?'Y':'N');
 	printf("Can get channel:\t%c\n",caps->get_channel!=NULL?'Y':'N');
-#ifdef WANT_OLD_VFO_TO_BE_REMOVED
-	printf("Can ctl mem/vfo:\t%c\n",caps->mv_ctl!=NULL?'Y':'N');
-#else
 	printf("Can ctl mem/vfo:\t%c\n",caps->vfo_op!=NULL?'Y':'N');
-#endif
+	printf("Can scan:\t%c\n",caps->scan!=NULL?'Y':'N');
 	printf("Can get info:\t%c\n",caps->get_info!=NULL?'Y':'N');
 	
 
@@ -501,29 +408,19 @@ int main (int argc, char *argv[])
 }
 
 
+static int print_ext(RIG *rig, const struct confparams *cfp, rig_ptr_t ptr)
+{
+	printf(" %s", cfp->name);
+
+	return 1;       /* process them all */
+}
+
+
 /*
  * NB: this function is not reentrant, because of the static buf.
  * 		but who cares?  --SF
  */
-static char *decode_modes(rmode_t modes)
-{
-	static char buf[80];
-
-	buf[0] = '\0';
-	if (modes&RIG_MODE_AM) strcat(buf,"AM ");
-	if (modes&RIG_MODE_CW) strcat(buf,"CW ");
-	if (modes&RIG_MODE_USB) strcat(buf,"USB ");
-	if (modes&RIG_MODE_LSB) strcat(buf,"LSB ");
-	if (modes&RIG_MODE_RTTY) strcat(buf,"RTTY ");
-	if (modes&RIG_MODE_FM) strcat(buf,"FM ");
-#ifdef RIG_MODE_WFM
-	if (modes&RIG_MODE_WFM) strcat(buf,"WFM ");
-#endif
-
-	return buf;
-}
-
-static const char *decode_mtype(enum chan_type_e type)
+static const char *decode_mtype(chan_type_t type)
 {
 		switch(type) {
 				case RIG_MTYPE_NONE: return "NONE";
@@ -591,9 +488,9 @@ int ts_sanity_check(const struct tuning_step_list tuning_step[])
 	last_ts = 0;
 	last_modes = RIG_MODE_NONE;
 	for (i=0; i<TSLSTSIZ; i++) {
-			if (tuning_step[i].modes == 0 && tuning_step[i].ts == 0)
+			if (RIG_IS_TS_END(tuning_step[i]))
 					break;
-			if (tuning_step[i].ts < last_ts && 
+			if (tuning_step[i].ts != RIG_TS_ANY && tuning_step[i].ts < last_ts && 
 							last_modes == tuning_step[i].modes)
 					return -1;
 			if (tuning_step[i].modes == 0)
@@ -608,3 +505,31 @@ int ts_sanity_check(const struct tuning_step_list tuning_step[])
 }
 
 
+static void dump_chan_caps(const channel_cap_t *chan)
+{
+  if (chan->bank_num) printf("BANK ");
+  if (chan->ant) printf("ANT ");
+  if (chan->freq) printf("FREQ ");
+  if (chan->mode) printf("MODE ");
+  if (chan->width) printf("WIDTH ");
+  if (chan->tx_freq) printf("TXFREQ ");
+  if (chan->tx_mode) printf("TXMODE ");
+  if (chan->tx_width) printf("TXWIDTH ");
+  if (chan->split) printf("SPLIT ");
+  if (chan->rptr_shift) printf("RPTRSHIFT ");
+  if (chan->rptr_offs) printf("RPTROFS ");
+  if (chan->tuning_step) printf("TS ");
+  if (chan->rit) printf("RIT ");
+  if (chan->xit) printf("XIT ");
+  if (chan->funcs) printf("FUNC ");
+  if (chan->levels) printf("LEVEL ");
+  if (chan->ctcss_tone) printf("TONE ");
+  if (chan->ctcss_sql) printf("CTCSS ");
+  if (chan->dcs_code) printf("DCSCODE ");
+  if (chan->dcs_sql) printf("DCSSQL ");
+  if (chan->scan_group) printf("SCANGRP ");
+  if (chan->flags) printf("FLAG ");    /* RIG_CHFLAG's */
+  if (chan->channel_desc) printf("NAME ");
+  if (chan->ext_levels) printf("EXTLVL ");
+}
+
