@@ -6,24 +6,24 @@
  * via serial interface to an FT-847 using the "CAT" interface.
  *
  *
- * $Id: ft847.c,v 1.13 2001/06/04 17:01:21 f4cfe Exp $  
+ * $Id: ft847.c,v 1.22 2003/04/07 22:42:05 fillods Exp $  
  *
  *
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- * 
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- * 
+ *  This library is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Lesser General Public
+ *  License as published by the Free Software Foundation; either
+ *  version 2 of the License, or (at your option) any later version.
+ *
+ *  This library is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *  Lesser General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Lesser General Public
+ *  License along with this library; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+ *
  */
 
 
@@ -45,17 +45,15 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
-#include <stdio.h>   /* Standard input/output definitions */
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
-#include <fcntl.h>   /* File control definitions */
-#include <errno.h>   /* Error number definitions */
-#include <termios.h> /* POSIX terminal control definitions */
-#include <sys/ioctl.h>
 
-#include <hamlib/rig.h>
-#include <hamlib/riglist.h>
+#include "hamlib/rig.h"
 #include "serial.h"
 #include "yaesu.h"
 #include "ft847.h"
@@ -161,8 +159,8 @@ static const yaesu_cmd_set_t ncmd[] = {
  */
 
 
-#define FT847_ALL_RX_MODES (RIG_MODE_AM|RIG_MODE_CW|RIG_MODE_USB|RIG_MODE_LSB|RIG_MODE_RTTY|RIG_MODE_FM)
-#define FT847_SSB_CW_RX_MODES (RIG_MODE_CW|RIG_MODE_USB|RIG_MODE_LSB)
+#define FT847_ALL_RX_MODES (RIG_MODE_AM|RIG_MODE_CW|RIG_MODE_SSB|RIG_MODE_FM)
+#define FT847_SSB_CW_RX_MODES (RIG_MODE_CW|RIG_MODE_SSB)
 #define FT847_AM_FM_RX_MODES (RIG_MODE_AM|RIG_MODE_FM)
 
 /* tx doesn't have WFM.
@@ -170,7 +168,7 @@ static const yaesu_cmd_set_t ncmd[] = {
  * 50W in 2m/70cm (12.5 watts AM carrier)
  */ 
 
-#define FT847_OTHER_TX_MODES (RIG_MODE_AM|RIG_MODE_CW|RIG_MODE_USB|RIG_MODE_LSB|RIG_MODE_RTTY|RIG_MODE_FM)
+#define FT847_OTHER_TX_MODES (RIG_MODE_AM|RIG_MODE_CW|RIG_MODE_SSB|RIG_MODE_FM)
 #define FT847_AM_TX_MODES (RIG_MODE_AM)
 
 #define FT847_FUNC_ALL (RIG_FUNC_FAGC|RIG_FUNC_NB|RIG_FUNC_COMP|RIG_FUNC_VOX|RIG_FUNC_TONE|RIG_FUNC_TSQL|RIG_FUNC_SBKIN|RIG_FUNC_FBKIN)	/* fix */
@@ -183,59 +181,60 @@ static const yaesu_cmd_set_t ncmd[] = {
 
 
 const struct rig_caps ft847_caps = {
-rig_model: RIG_MODEL_FT847,
-model_name:"FT-847", 
-mfg_name: "Yaesu", 
-version: "0.1", 
-copyright: "GPL",
-status: RIG_STATUS_ALPHA,
-rig_type: RIG_TYPE_TRANSCEIVER,
-ptt_type: RIG_PTT_RIG,
-dcd_type: RIG_DCD_NONE,
-port_type: RIG_PORT_SERIAL,
-serial_rate_min: 4800,
-serial_rate_max: 57600,
-serial_data_bits: 8,
-serial_stop_bits: 2,
-serial_parity: RIG_PARITY_NONE,
-serial_handshake: RIG_HANDSHAKE_NONE, 
-write_delay: FT847_WRITE_DELAY,
-post_write_delay: FT847_POST_WRITE_DELAY,
-timeout: 100,
-retry: 0, 
+.rig_model =  RIG_MODEL_FT847,
+.model_name = "FT-847", 
+.mfg_name =  "Yaesu", 
+.version =  "0.1", 
+.copyright =  "LGPL",
+.status =  RIG_STATUS_ALPHA,
+.rig_type =  RIG_TYPE_TRANSCEIVER,
+.ptt_type =  RIG_PTT_RIG,
+.dcd_type =  RIG_DCD_NONE,
+.port_type =  RIG_PORT_SERIAL,
+.serial_rate_min =  4800,
+.serial_rate_max =  57600,
+.serial_data_bits =  8,
+.serial_stop_bits =  2,
+.serial_parity =  RIG_PARITY_NONE,
+.serial_handshake =  RIG_HANDSHAKE_NONE, 
+.write_delay =  FT847_WRITE_DELAY,
+.post_write_delay =  FT847_POST_WRITE_DELAY,
+.timeout =  100,
+.retry =  0, 
 
-has_get_func: RIG_FUNC_NONE,
-has_set_func: FT847_FUNC_ALL, 
-has_get_level: RIG_LEVEL_NONE,	/* FIXME! */
-has_set_level: RIG_LEVEL_NONE,
-has_get_parm: RIG_PARM_NONE,
-has_set_parm: RIG_PARM_NONE,	/* FIXME: parms */
-level_gran: {}, 		/* granularity */
-parm_gran: {},
-ctcss_list: NULL,	/* FIXME: CTCSS/DCS list */
-dcs_list: NULL,
-preamp:  { RIG_DBLST_END, },	/* FIXME! */
-attenuator:  { RIG_DBLST_END, },
-max_rit: Hz(9999),
-max_xit: Hz(0),
-max_ifshift: Hz(0),
-targetable_vfo: 1,
-transceive: RIG_TRN_OFF,
-bank_qty:  0,
-chan_desc_sz: 0,
+.has_get_func =  RIG_FUNC_NONE,
+.has_set_func =  FT847_FUNC_ALL, 
+.has_get_level =  RIG_LEVEL_NONE,	/* FIXME! */
+.has_set_level =  RIG_LEVEL_NONE,
+.has_get_parm =  RIG_PARM_NONE,
+.has_set_parm =  RIG_PARM_NONE,	/* FIXME: parms */
+.level_gran =  {}, 		/* granularity */
+.parm_gran =  {},
+.ctcss_list =  NULL,	/* FIXME: CTCSS/DCS list */
+.dcs_list =  NULL,
+.preamp =   { RIG_DBLST_END, },	/* FIXME! */
+.attenuator =   { RIG_DBLST_END, },
+.max_rit =  Hz(9999),
+.max_xit =  Hz(0),
+.max_ifshift =  Hz(0),
+.targetable_vfo =  RIG_TARGETABLE_FREQ,
+.transceive =  RIG_TRN_OFF,
+.bank_qty =   0,
+.chan_desc_sz =  0,
 
-chan_list: { RIG_CHAN_END, },	/* FIXME: memory chan list: 78 */
+.chan_list =  { RIG_CHAN_END, },	/* FIXME: memory chan list: 78 */
 
-rx_range_list1: { RIG_FRNG_END, },    /* FIXME: enter region 1 setting */
-tx_range_list1: { RIG_FRNG_END, },
-rx_range_list2: 
-  { {kHz(100),MHz(76),FT847_ALL_RX_MODES,-1,-1}, /* rx range begin */
+.rx_range_list1 =  { RIG_FRNG_END, },    /* FIXME: enter region 1 setting */
+.tx_range_list1 =  { RIG_FRNG_END, },
+.rx_range_list2 =  
+  { {kHz(100),MHz(30),FT847_ALL_RX_MODES,-1,-1}, /* rx range begin */
+    {MHz(36),MHz(76),FT847_ALL_RX_MODES,-1,-1},
     {MHz(108),MHz(174),FT847_ALL_RX_MODES,-1,-1},
     {MHz(420),MHz(512),FT847_ALL_RX_MODES,-1,-1},
 
     RIG_FRNG_END, }, /* rx range end */
 
-tx_range_list2:
+.tx_range_list2 = 
   { {MHz(1.8),1999999,FT847_OTHER_TX_MODES,W(5),W(100)},	/* 5-100W class */
     {MHz(1.8),1999999,FT847_AM_TX_MODES,W(1),W(25)},	/* 1-25W class */
 
@@ -274,7 +273,7 @@ tx_range_list2:
 
     RIG_FRNG_END, },
 
-tuning_steps: { {FT847_SSB_CW_RX_MODES,1}, /* normal */
+.tuning_steps =  { {FT847_SSB_CW_RX_MODES,1}, /* normal */
     {FT847_SSB_CW_RX_MODES,10}, /* fast */
     {FT847_SSB_CW_RX_MODES,100}, /* faster */
 
@@ -284,26 +283,30 @@ tuning_steps: { {FT847_SSB_CW_RX_MODES,1}, /* normal */
         
     RIG_TS_END,
   },  
-      /* mode/filter list, remember: order matters! */
-filters: {
-		/* FIXME! */
+      /* mode/filter list, .remember =  order matters! */
+.filters =  {
+		{RIG_MODE_SSB|RIG_MODE_CW, kHz(2.2)},
+		{RIG_MODE_AM, kHz(9)},
+		{RIG_MODE_AM, kHz(2.2)},
+		{RIG_MODE_FM, kHz(15)},
+		{RIG_MODE_FM, kHz(9)},
 		RIG_FLT_END,
   },
 
-priv:  NULL,
-rig_init:  ft847_init, 
-rig_cleanup: ft847_cleanup, 
-rig_open:  ft847_open, 
-rig_close: ft847_close, 
+.priv =   NULL,
+.rig_init =   ft847_init, 
+.rig_cleanup =  ft847_cleanup, 
+.rig_open =   ft847_open, 
+.rig_close =  ft847_close, 
 
-set_freq:  ft847_set_freq,		/* set freq */
-get_freq: ft847_get_freq,		/* get freq */
-set_mode: ft847_set_mode,		/* set mode */
-get_mode: ft847_get_mode,		/* get mode */
-set_vfo: ft847_set_vfo,		/* set vfo */
-get_vfo: ft847_get_vfo,		/* get vfo */
-set_ptt: ft847_set_ptt,		/* set ptt */
-get_ptt: ft847_get_ptt,		/* get ptt */
+.set_freq =   ft847_set_freq,		/* set freq */
+.get_freq =  ft847_get_freq,		/* get freq */
+.set_mode =  ft847_set_mode,		/* set mode */
+.get_mode =  ft847_get_mode,		/* get mode */
+.set_vfo =  ft847_set_vfo,		/* set vfo */
+.get_vfo =  ft847_get_vfo,		/* get vfo */
+.set_ptt =  ft847_set_ptt,		/* set ptt */
+.get_ptt =  ft847_get_ptt,		/* get ptt */
 
 };
 
@@ -449,7 +452,7 @@ int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq) {
 
   rig_s = &rig->state;
 
-  rig_debug(RIG_DEBUG_VERBOSE,"ft847: requested freq = %Li Hz \n", freq);
+  rig_debug(RIG_DEBUG_VERBOSE,"ft847: requested freq = %lli Hz \n", freq);
 
 
   /* 
@@ -484,7 +487,7 @@ int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq) {
   to_bcd_be(p->p_cmd,freq/10,8);	/* store bcd format in in p_cmd */
 				/* TODO -- fix 10Hz resolution -- FS */
 
-  rig_debug(RIG_DEBUG_VERBOSE,"ft847: requested freq after conversion = %Li Hz \n", from_bcd_be(p->p_cmd,8)* 10 );
+  rig_debug(RIG_DEBUG_VERBOSE,"ft847: requested freq after conversion = %lli Hz \n", from_bcd_be(p->p_cmd,8)* 10 );
 
   cmd = p->p_cmd; /* get native sequence */
   write_block(&rig_s->rigport, cmd, YAESU_CMD_LENGTH);
@@ -492,9 +495,115 @@ int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq) {
   return RIG_OK;
 }
 
-int ft847_get_freq(RIG *rig, vfo_t vfo, freq_t *freq) {
+#define MD_LSB  0x00
+#define MD_USB  0x01
+#define MD_CW   0x02
+#define MD_CWR  0x03
+#define MD_AM   0x04
+#define MD_FM   0x08
+#define MD_CWN  0x82
+#define MD_CWNR 0x83
+#define MD_AMN  0x84
+#define MD_FMN  0x88
 
-  return -RIG_ENIMPL;
+static int get_freq_and_mode(RIG *rig, vfo_t vfo, freq_t *freq, rmode_t *mode,
+				pbwidth_t *width) {
+  struct rig_state *rs = &rig->state;
+  struct ft847_priv_data *p;
+  unsigned char *cmd;		/* points to sequence to send */
+  unsigned char cmd_index;	/* index of sequence to send */
+  unsigned char data[8];
+  int n;
+
+  p = (struct ft847_priv_data*)rs->priv;
+
+  rig_debug(RIG_DEBUG_VERBOSE,"ft847: vfo =%i \n", vfo);
+  
+  if (vfo == RIG_VFO_MAIN)
+		  vfo = p->current_vfo;
+
+  /*
+   * TODO:
+   *	FT_847_NATIVE_CAT_GET_FREQ_MODE_STATUS_SAT_RX
+   *	FT_847_NATIVE_CAT_GET_FREQ_MODE_STATUS_SAT_TX
+   */
+  switch(vfo) {
+  case RIG_VFO_MAIN:
+    cmd_index = FT_847_NATIVE_CAT_GET_FREQ_MODE_STATUS_MAIN;
+    break;
+
+  default:
+    rig_debug(RIG_DEBUG_VERBOSE,"ft847: Unknown  VFO \n");
+    return -RIG_EINVAL;		/* sorry, wrong VFO */
+  }
+
+  memcpy(p->p_cmd,&ncmd[cmd_index].nseq,YAESU_CMD_LENGTH);  
+
+  cmd = p->p_cmd;
+  write_block(&rs->rigport, cmd, YAESU_CMD_LENGTH);
+
+  n = read_block(&rs->rigport, data, YAESU_CMD_LENGTH);
+  if (n != YAESU_CMD_LENGTH) {
+  		rig_debug(RIG_DEBUG_ERR,"ft847: read_block returned %d\n", n);
+		return n < 0 ? n : -RIG_EPROTO;
+  }
+
+	/* Remember, this is 10Hz resolution */
+  *freq = 10*from_bcd_be(data, 8);
+
+  *width = RIG_PASSBAND_NORMAL;
+  switch (data[4]) {
+  case MD_LSB: *mode = RIG_MODE_LSB; break;
+  case MD_USB: *mode = RIG_MODE_USB; break;
+
+  case MD_CWN: 
+			   *width = rig_passband_narrow(rig, RIG_MODE_CW);
+  case MD_CW:  
+			   *mode = RIG_MODE_CW;
+			   break;
+
+#ifdef RIG_MODE_CWR
+  case MD_CWNR:
+			   *width = rig_passband_narrow(rig, RIG_MODE_CW);
+  case MD_CWR: 
+				*mode = RIG_MODE_CWR;
+				break;
+#endif
+
+  case MD_AMN:
+			   *width = rig_passband_narrow(rig, RIG_MODE_AM);
+  case MD_AM:
+			   *mode = RIG_MODE_AM;
+			   break;
+
+  case MD_FMN:
+			   *width = rig_passband_narrow(rig, RIG_MODE_FM);
+  case MD_FM:  
+			   *mode = RIG_MODE_FM; 
+			   break;
+  default:
+    *mode = RIG_MODE_NONE;
+    rig_debug(RIG_DEBUG_VERBOSE,"ft847: Unknown mode %02x\n", data[4]);
+  }
+  if (*width == RIG_PASSBAND_NORMAL)
+		  *width = rig_passband_normal(rig, *mode);
+
+  return RIG_OK;
+}
+
+/*
+ * Note taken from http://my.en.com/~rayd/ft-847/FAQ/page4.htm#pollingcodes
+ * The FT-847, as originally delivered, could not poll the radio for frequency
+ * and mode information. This was added beginning with the 8G05 production
+ * runs. The Operating Manual does not show the codes for polling the radio.
+ * Note that you cannot query the sub-VFO, nor can you swap VFOs via software.
+ */
+int ft847_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
+{
+  rmode_t mode;
+  pbwidth_t width;
+
+  return get_freq_and_mode(rig, vfo, freq, &mode, &width);
 }
 
 
@@ -539,16 +648,7 @@ int ft847_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width) {
    * Now set width
    */
 
-  switch(width) {
-  case RIG_PASSBAND_NORMAL:	         /* easy case , no change to native sequence */
-    ft847_send_priv_cmd(rig,cmd_index);	 /* TODO -- check return codes */
-    return RIG_OK;		         /* I am out of here .. */
-
-#ifdef RIG_PASSBAND_OLDTIME
-  case RIG_PASSBAND_WIDE:
-    return -RIG_EINVAL;		/* sorry, WIDE WIDTH is not supported */
-
-  case RIG_PASSBAND_NARROW:	/* must set narrow */
+  if (width == rig_passband_narrow(rig, mode)) {
     switch(mode) {
     case RIG_MODE_AM:
       cmd_index = FT_847_NATIVE_CAT_SET_MODE_MAIN_AMN;
@@ -562,16 +662,12 @@ int ft847_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width) {
     default:
       return -RIG_EINVAL;		/* sorry, wrong MODE/WIDTH combo  */    
     }
-    break;
-#else
-	/* TODO ! */
-#endif
-
-  default:
-    return -RIG_EINVAL;		/* sorry, wrong WIDTH requested  */    
-  }
-  
-
+  } else {
+  		if (width != RIG_PASSBAND_NORMAL && 
+						width != rig_passband_normal(rig, mode)) {
+      		return -RIG_EINVAL;		/* sorry, wrong MODE/WIDTH combo  */    
+		}
+	}
   /*
    * Now send the command
    */
@@ -582,7 +678,9 @@ int ft847_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width) {
 }
 
 int ft847_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width) {
-  return -RIG_ENIMPL;
+  freq_t freq;
+
+  return get_freq_and_mode(rig, vfo, &freq, mode, width);
 }
 
 
